/*----------------------------------------------------------------------
      File name:      ggrads_ctl.c    for swp2ga

         fout        : GrADS output file name
         nx,ny,nz    : dimensions of x,y, and z
         x/y/zmin    : min. values of x,y,z 
         dx,dy,dz    : grid intervals of x,y, and z

	                     ver. 1.0   Sep. 16, 1999  by Shin SATOH
----------------------------------------------------------------------*/
# include "Volume.h"
# include "RadarDesc.h"
# include "Parameter.h"
# include "Ray.h"
# include "dd_defines.h"
# include "dd_general_info.h"
# include "input_sweepfiles.h"
#include <stdio.h>
#include <string.h>

int ggrads_ctl (dgi,dis,fout,np_out,np_act,nx,ny,nz,
                xmin,ymin,zmin,dx,dy,dz,no_data)
    struct dd_general_info *dgi;
    struct dd_input_sweepfiles_v3 *dis;
    char   *fout;
    int    *np_out;
    int    np_act,nx,ny,nz;
    float  xmin,ymin,zmin,dx,dy,dz,no_data;
{

    DDS_PTR dds=dgi->dds;
    FILE         *fpc;
    static char  c_name[260],f_name[64],c_title[64],p_unit[9];
    static char  p_var[MAX_PARMS][9],p_descr[MAX_PARMS][51];
    int          i,j,k,kk,kmax,len,np;
    int          ihr,imin,iday,imon,iyr;
    static char  *cmon[12]={"JAN","FEB","MAR","APR","MAY","JUN",
                            "JUL","AUG","SEP","OCT","NOV","DEC"};

/* Open GrADS control file */
    strcpy(c_name,fout);  
    strcat(c_name,".ctl");
    if((fpc = fopen(c_name,"w" )) == NULL ){
       printf("File cannot open ! : %s \n",c_name); 
       return(1);
    }

/* Get GrADS file name without path */
    strcpy(f_name,fout);
    len = strlen(fout) - 1;
    for(i=0; i<len; i++){
       if(strncmp(&fout[i],"/",1)==0) strcpy(f_name,&fout[i+1]);
    }

/* Get time value for TDEF */
    iyr  = (int)dds->vold->year;
    imon = (int)dds->vold->month;
    iday = (int)dds->vold->day;
    ihr  = (int)dds->ryib->hour;    /* ihr is the end time */
    imin = (int)dds->ryib->minute;  /* imin is the end time */

    str_terminate(c_title, dds->radd->radar_name, 8);
    strcat(c_title," (data in RT coordinates made by swp2ga)");

/* Get variable names for VARS */
    i=0;
    kmax = dgi->source_num_parms;
    for(kk=0; kk<kmax; kk++){
      k=-99;
      for (np=0; np<np_act; np++){
         if(np_out[np]==kk) k=kk;
      }
      if(k>=0){
         str_terminate(&p_var[i][0], dds->parm[k]->parameter_name, 8);
         len = strlen(&p_var[i][0]);
         for(j=len; j<8; j++) {strncpy(&p_var[i][j]," ",1);}
         strncpy(&p_var[i][8],"\0",1);
# ifdef DEBUG
         printf("len=%d  p_var=%c%c%c%c%c%c%c%c%c\n",len,
         p_var[i][0],p_var[i][1],p_var[i][2],p_var[i][3],
         p_var[i][4],p_var[i][5],p_var[i][6],p_var[i][7],p_var[i][8]);
# endif
         str_terminate(&p_descr[i][0], dds->parm[k]->param_description, 40);
         str_terminate(p_unit, dds->parm[k]->param_units, 8);
         strcat(&p_descr[i][0]," [");
         strcat(&p_descr[i][0],p_unit);
         strcat(&p_descr[i][0],"]");
         i++;
      }
    }

/* output GrADS control file */
    fprintf(fpc,"DSET    ^%s\n",f_name);
    fprintf(fpc,"TITLE    %s\n",c_title);
    fprintf(fpc,"UNDEF    %8.2f\n",no_data);
    fprintf(fpc,"XDEF     %5d LINEAR  %8.3f  %8.5f\n",nx,xmin,dx);
    fprintf(fpc,"YDEF     %5d LINEAR  %8.3f  %8.5f\n",ny,ymin,dy);
    fprintf(fpc,"ZDEF     %5d LINEAR  %8.3f  %8.5f\n",nz,zmin,dz);
    fprintf(fpc,"TDEF     %5d LINEAR  %2.2d:%2.2dZ%2.2d%3s%4d  %dmn\n",
                           1,ihr,imin,iday,cmon[imon+1],iyr,1);
    fprintf(fpc,"VARS      %d\n",np_act);
    for (i=0; i < np_act; i++){
       fprintf(fpc,"%s     %d   99   %s\n",&p_var[i][0],nz,&p_descr[i][0]);
    }
    fprintf(fpc,"ENDVARS\n");
    fclose(fpc);

    return(0);
}
