/*------------------------------------------------------------------------
     print_info.c
     * This file contains the following routines
         print_header_info(dgi,dis)
         print_params_info(dgi)
                                            by Shin Satoh  Swp. 10, 1999
  ------------------------------------------------------------------------*/
# include "Volume.h"
# include "Comment.h"
# include "RadarDesc.h"
# include "Correction.h"
# include "Parameter.h"
# include "CellVector.h"
# include "CellSpacing.h"
# include "Sweep.h"
# include "Platform.h"
# include "Ray.h"
# include "Pdata.h"
# include "Qdata.h"
# include "FieldParam.h"
# include "super_SWIB.h"

# include "dd_defines.h"
# include "dorade_share.h"
# include "dd_general_info.h"

# include "dd_files.h"
# include "input_limits.h"
# include "input_sweepfiles.h"
# include "dd_stats.h"
# include "function_decl.h"
# include "dgi_func_decl.h"


int print_header_info(dgi,dis)
    struct dd_general_info *dgi;
    struct dd_input_sweepfiles_v3 *dis;
{
    DDS_PTR dds=dgi->dds;
    DD_TIME *dts=dds->dts;
    char str[128];
    int  ii;

    printf("\n===== dis: dd_input_sweepfiles_v3 =====\n");
    printf("  MB=%f  edit=%d  file_count=%d  num_radars=%d  num_swps=%d \n",
            dis->MB,dis->editing,dis->file_count,dis->num_radars,dis->num_swps);
    printf("  ray_count=%d  rec_count=%d  sweep_count=%d  vel_count=%d \n",
            dis->ray_count,dis->rec_count,dis->sweep_count,dis->vol_count);

    printf("\n===== usi: unique_sweepfile_info_v3 =====\n");
    printf("  num_rays=%d  num_swps=%d  version=%d  vol_num=%d \n",
            dis->usi_top->num_rays,dis->usi_top->num_swps,dis->usi_top->version,
            dis->usi_top->vol_num);
    printf("  directory=%s \n",dis->usi_top->directory);
    printf("  filename =%s \n",dis->usi_top->filename);
    printf("  radarname=%s \n",dis->usi_top->radar_name);

    printf("\n===== VOLD: volume descriptor =====\n");
    printf("  format_version=%d  volume_num=%d  maximum_bytes=%d \n",
            dds->vold->format_version,dds->vold->volume_num,dds->vold->maximum_bytes);
    printf("  proj_name=%s \n",dds->vold->proj_name);
    printf("  data_set_date=%4d/%2d/%2d,%2d:%2d:%2d \n",
            dds->vold->year,dds->vold->month,dds->vold->day,dds->vold->data_set_hour,
            dds->vold->data_set_minute,dds->vold->data_set_second);
    str_terminate(str, dds->vold->flight_num, 8);
    printf("  flight_num=%s \n",str);
    str_terminate(str, dds->vold->gen_facility, 8);
    printf("  facility generated this rec=%s \n",str);
    printf("  date generated this rec=%2d/%2d/%2d \n",
            dds->vold->gen_year,dds->vold->gen_month,dds->vold->gen_day);
    printf("  number_sensor_descriptor=%d \n",dds->vold->number_sensor_des);

    printf("\n===== RADD: radar_d =====\n");
    printf("  Identifier for a radar descriptor=%d\n",dds->radd->radar_des_length);
    str_terminate(str, dds->radd->radar_name, 8);
    printf("  radar_name=%s \n",str);
    printf("  radar_const=%f  peak_power=%f[kW]  noise_power=%f[dBm]\n",
            dds->radd->radar_const,dds->radd->peak_power,dds->radd->noise_power);
    printf("  receiver_gain=%f[dB]  antenna_gain=%f[dB]  system_gain=%f[dB]\n",
            dds->radd->receiver_gain,dds->radd->antenna_gain,dds->radd->system_gain);
    printf("  beam_width(horz)=%f[deg]  beam_width(vert)=%f[deg]\n",
            dds->radd->horz_beam_width,dds->radd->vert_beam_width);
    if(dds->radd->radar_type==0) printf("  Ground radar\n");
    if(dds->radd->radar_type==1) printf("  Airborn Fore radar\n");
    if(dds->radd->radar_type==2) printf("  Airborn Aft radar\n");
    if(dds->radd->radar_type==3) printf("  Airborn Tail radar\n");
    if(dds->radd->radar_type==4) printf("  Airborn Lower Fuselage radar\n");
    if(dds->radd->radar_type==5) printf("  Shipborn radar\n");
    if(dds->radd->scan_mode==0) printf("  Scan mode= Calibration\n");
    if(dds->radd->scan_mode==1) printf("  Scan mode= PPI\n");
    if(dds->radd->scan_mode==2) printf("  Scan mode= COPLANE\n");
    if(dds->radd->scan_mode==3) printf("  Scan mode= RHI\n");
    if(dds->radd->scan_mode==4) printf("  Scan mode= Vertical Pointing\n");
    if(dds->radd->scan_mode==5) printf("  Scan mode= Target (Stationary)\n");
    if(dds->radd->scan_mode==6) printf("  Scan mode= Manual\n");
    if(dds->radd->scan_mode==7) printf("  Scan mode= Idle (Out of Control)\n");
    printf("  ant_rotation(req)=%f[deg/sec]  scan_mode_pram0/1=%f/%f\n",
           dds->radd->req_rotat_vel,dds->radd->scan_mode_pram0,dds->radd->scan_mode_pram1);
    printf("  num_of_param=%d  num_of_additional_descriptor_block=%d\n",
           dds->radd->num_parameter_des,dds->radd->total_num_des);
    if(dds->radd->data_compress==0) printf("  data_compress=NONE\n");
    if(dds->radd->data_compress==1) printf("  data_compress=HRD\n");
    if(dds->radd->data_reduction==1) printf("  data_reduction_algor=NONE");
    if(dds->radd->data_reduction==2) printf("  data_reduction_algor=Between two angles");
    if(dds->radd->data_reduction==3) printf("  data_reduction_algor=Between circles");
    if(dds->radd->data_reduction==4) printf("  data_reduction_algor=Above/Below alti");
    if(dds->radd->data_reduction>=1 && dds->radd->data_reduction<=4)
      printf("  data_red_parm0/1=%f/%f\n",dds->radd->data_red_parm0,dds->radd->data_red_parm1);
    printf("  longitude=%f  latitude=%f  altitude=%f[km above MSL]\n",
           dds->radd->radar_longitude,dds->radd->radar_latitude,dds->radd->radar_altitude);
    printf("  effective_unambiguous_vel=%f[m/s]  unambiguous_range=%f[km]\n",
           dds->radd->eff_unamb_vel,dds->radd->eff_unamb_range);
    printf("  num_of_freq_transmitted=%d  num_of_interpulse_periods=%d\n",
           dds->radd->num_freq_trans,dds->radd->num_ipps_trans);
    printf("  frequency=%f,%f,%f,%f,%f\n",dds->radd->freq1,dds->radd->freq2,
           dds->radd->freq3,dds->radd->freq4,dds->radd->freq5);
    printf("  interpulse_period=%f,%f,%f,%f,%f\n",dds->radd->interpulse_per1,
           dds->radd->interpulse_per2,dds->radd->interpulse_per3,
           dds->radd->interpulse_per4,dds->radd->interpulse_per5);
    printf("  extension_num=%d\n",dds->radd->extension_num);
    str_terminate(str, dds->radd->config_name, 8);
    printf("  config_name=%s  config_num=%d\n",str,dds->radd->config_num);
    printf("  pulth_witdh=%f[micro sec]\n",dds->radd->pulsewidth);
    printf("  coplane baselines=%f,%f\n",
           dds->radd->primary_cop_baseln,dds->radd->secondary_cop_baseln);
    printf("  pulse compression bandwidth/waveform_type=%f/%d\n",
           dds->radd->pc_xmtr_bandwidth,dds->radd->pc_waveform_type);
    str_terminate(str, dds->radd->site_name, 20);
    printf("  site_name=%s \n",str);

    printf("\n===== CELV: cell_d =====\n");
    printf("  descriptor_length=%d  number of cells=%d\n",
           dds->celv->cell_des_len,dds->celv->number_cells);
    printf("  distance from the radar to each cell[m]=%8.3f,%8.3f,%8.3f\n",
           dds->celv->dist_cells[0],dds->celv->dist_cells[1],dds->celv->dist_cells[2]);
    ii = dds->celv->number_cells;
    printf("           %8.3f,%8.3f %8.3f ------ %10.3f,%10.3f\n",
           dds->celv->dist_cells[3],dds->celv->dist_cells[4],dds->celv->dist_cells[5],
           dds->celv->dist_cells[ii-2],dds->celv->dist_cells[ii-1]);

    printf("\n===== CFAC: correction_d =====\n");
    printf("  correction descriptor length=%d\n",dds->cfac->correction_des_length); 
    printf("  AZ_corr=%f[deg]  EL_corr=%f[deg]  range_deray_corr=%f[m]\n",
           dds->cfac->azimuth_corr,dds->cfac->elevation_corr,dds->cfac->range_delay_corr);
    printf("  LON_corr=%f  LAT_corr=%f\n",
           dds->cfac->longitude_corr,dds->cfac->latitude_corr);
    printf("  pres_alt_corr=%f[km(MSL)]  radar_alt_corr=%f[km(AGL)]\n",
           dds->cfac->radar_alt_corr,dds->cfac->pressure_alt_corr);
    if(dds->cfac->ew_gndspd_corr != 0.0 || dds->cfac->ns_gndspd_corr != 0.0){
       printf("  platform_ground_speed_corr: E-W=%f  N-S=%f  Vert=%f[m/s]\n",
              dds->cfac->ew_gndspd_corr,dds->cfac->ns_gndspd_corr,dds->cfac->vert_vel_corr);  
    }
    if(dds->cfac->roll_corr != 0.0 || dds->cfac->pitch_corr != 0.0){
       printf("  platform_angle_corr: heading=%f  roll=%f  pitch=%f[deg]\n",
              dds->cfac->heading_corr,dds->cfac->roll_corr,dds->cfac->pitch_corr);
       printf("                      : drift=%f  rot_angle=%f  tilt=%f[deg]\n",
              dds->cfac->drift_corr,dds->cfac->rot_angle_corr,dds->cfac->tilt_corr);
    }

    printf("\n===== SWIB: sweepinfo_d =====\n"); 
    printf("  sweep info descriptor length=%d\n",dds->swib->sweep_des_length); 
    str_terminate(str, dds->swib->radar_name, 8);
    if(str[0] != '\0') { 
      printf("  radar_name=%s \n",str);
    }
    printf("  sweep_num=%d  num_of_rays=%d\n",dds->swib->sweep_num,dds->swib->num_rays);
    printf("  start_angle=%f[deg]  stop_angle=%f[deg]  fixed_angle=%f[deg]\n",
           dds->swib->start_angle,dds->swib->stop_angle,dds->swib->fixed_angle);
    if(dds->swib->filter_flag != 0){
      printf("  filter_flag=%d\n",dds->swib->filter_flag);
    }
    printf("\n");

    return(0);
}
/* c------------------------------------------------------------------------ */

int print_params_info(dgi)
    struct dd_general_info *dgi;
{
    DDS_PTR dds=dgi->dds;
    char str[128];
    float fff;
    int   pn;

    for(pn=0; pn < dgi->source_num_parms; pn++) {

    /* paramers in dds->param[] */
    printf("===== PARM: parameter_d  No=%2d  =====\n",pn);
    str_terminate(str, dds->parm[pn]->parameter_name, 8);
    printf("  parameter_name=%s \n",str);
    str_terminate(str, dds->parm[pn]->param_description, 40);
    printf("  parameter_description=%s \n",str);
    str_terminate(str, dds->parm[pn]->param_units, 8);
    printf("  parameter_units=%s \n",str);

    printf("  ipp_used=%d  freq_used=%d  receiver_bandwidth=%f[MHz]\n",
           dds->parm[pn]->interpulse_time,dds->parm[pn]->xmitted_freq,
           dds->parm[pn]->recvr_bandwidth);
    if(dds->parm[pn]->pulse_width != 0 || dds->parm[pn]->num_samples != 0) {
      printf("  pulse_width=%d[m]  num_of_samples=%d  \n",
             dds->parm[pn]->pulse_width,dds->parm[pn]->num_samples);
    }
    if(dds->parm[pn]->polarization==0) printf("  Horizontal polarization  binary_format=%d\n",
       dds->parm[pn]->binary_format);
    if(dds->parm[pn]->polarization==1) printf("  Vertical polarization    binary_format=%d\n",
       dds->parm[pn]->binary_format);
    if(dds->parm[pn]->polarization==2) printf("  Circular polarization    binary_format=%d\n",
       dds->parm[pn]->binary_format);
    if(dds->parm[pn]->polarization==3) printf("  Elliptical polarization  binary_format=%d\n",
       dds->parm[pn]->binary_format);
    str_terminate(str, dds->parm[pn]->threshold_field, 8);
    if(str[0] != '\0') { 
      printf("  name_thresholded(this param)=%s \n",str);
    }
    printf("  threshold_value=%10.2f  parameter_scale=%8.2f  parameter_bias=%8.2f\n",
           dds->parm[pn]->threshold_value,dds->parm[pn]->parameter_scale,
           dds->parm[pn]->parameter_bias);
    printf("  bad_data=%d  extension_num=%d\n",
           dds->parm[pn]->bad_data,dds->parm[pn]->extension_num);
    str_terminate(str, dds->parm[pn]->config_name, 8);
    if(str[0] != '\0' || dds->parm[pn]->config_num != 0) {
       printf("  config_name=%s  config_num=%d\n",str,dds->parm[pn]->config_num);
    }
    if(dds->parm[pn]->offset_to_data != 0 || dds->parm[pn]->mks_conversion != 0.0) {
       printf("  offset_to_data=%d[byte]  mks_conv=%f\n",
              dds->parm[pn]->offset_to_data,dds->parm[pn]->mks_conversion);
    }
    str_terminate(str, dds->parm[pn]->qdata_names, 32);
    if(str[0] != '\0' || dds->parm[pn]->num_qnames != 0) {
       printf("  qdata_names(8byte*4)=%s  num_qnames=%d\n",
              str,dds->parm[pn]->num_qnames);
    }
    str_terminate(str, dds->parm[pn]->criteria_names, 32);
    if(str[0] != '\0' || dds->parm[pn]->num_criteria != 0) {
       printf("  criteria_names(8byte*4)=%s  num_criteria=%d\n",
              str,dds->parm[pn]->num_criteria);
    }
    if(dds->parm[pn]->number_cells != 0.0 || dds->parm[pn]->eff_unamb_vel != 0.0) {
       printf("  num_cells=%d  eff_unamb_vel=%f[m/s]\n",
              dds->parm[pn]->number_cells,dds->parm[pn]->eff_unamb_vel);
    }
    if(dds->parm[pn]->meters_to_first_cell != 0.0 || 
       dds->parm[pn]->meters_between_cells != 0.0 ) {
       printf("  meters_to_first_cell=%f[m]  meters_between_cells=%f[m]\n",
              dds->parm[pn]->meters_to_first_cell,dds->parm[pn]->meters_between_cells);
    }

    /* array parameters in dds */
    printf("  field_id_num=%d", dds->field_id_num[pn]);
    printf("  field_present=%d", dds->field_present[pn]);
    printf("  last_cell_num=%d", dds->last_cell_num[pn]);
    printf("  sizeof_qdat=%d\n", dds->sizeof_qdat[pn]);

    }

    return(0);
};
/* c------------------------------------------------------------------------ */

int print_ray_info(dgi)
    struct dd_general_info *dgi;
{
    DDS_PTR dds=dgi->dds;

    if(dds->ryib->ray_status == 0){
      printf("#--> Find RYIB (%d byte):  sweep_num=%d\n",
             sizeof(struct ray_i),dds->ryib->sweep_num);
    }
    if(dds->ryib->ray_status == 1){
      printf("#--> Find RYIB (%d byte):  sweep_num=%d  <TRANSITION>\n",
             sizeof(struct ray_i),dds->ryib->sweep_num);
    }
    if(dds->ryib->ray_status == 2){
      printf("#--> Find RYIB (%d byte):  sweep_num=%d  <BAD STATUS>\n",
             sizeof(struct ray_i),dds->ryib->sweep_num);
    }
    printf("  TIME=%2.2d:%2.2d:%2.2d.%3.3d (%d julian_day)\n",
            dds->ryib->hour,dds->ryib->minute,dds->ryib->second,
            dds->ryib->millisecond,dds->ryib->julian_day);
    printf("  AZ=%f  EL=%f  Power=%f[kW]  Scan=%f[deg/s]\n",
            dds->ryib->azimuth,dds->ryib->elevation,
            dds->ryib->peak_power,dds->ryib->true_scan_rate);  
    return(0);
};
