C**********************************************************************
C  dow2d.f
C
C  This program is to convert R-T data (result of swp2ga) 
C  into X-Y data (2D Cartesian-space PPI) using nearest point method
C  for OU DOW
C                                  Sep. 17, 1999   by Shinsuke Satoh
C
C  INPUT file:  R-T data in 4-byte real (result from swp2ga)
C               to read 6 parameters (VE DM NCP SW DZ DCZ)
C  OUTPUT file: X-Y data of VE DM NCP SW DZ DCZ in 4-byte real
C               GrADS ctl file (xxxxxx.ctl)
C  ALGORITHM: coodinate convertion using a nearest point
C          
C (Parameter)
C    IMAX      : Num. of cells (X dimension size, output_size-3)
C    JMAX      : Num. of rays  (Y dimension size, output_size-1)
C    KMAX      : Num. of parameters
C    XYRESO    : Horizontal grid size(km)
C    XYRNG     : Output domain size (km)
C    XMIN,YMIN : First X/Y range(km) in the output domain
C    XC,YC     : Location of the radar site, Distance(km) from X/YMIN
C    BEAM_W    : Horizontal beam width(deg.)
C
C**********************************************************************
C*** Ser Parameters ****
      PARAMETER (IMAX=412)
      PARAMETER (JMAX=1596)
      PARAMETER (KMAX=6)
      PARAMETER (XYRESO=1.0)
      PARAMETER (XYRNG=160.0,XMIN=-80.0,YMIN=-80.0)
      PARAMETER (XC=0.0-XMIN,YC=0.0-YMIN)                              
      PARAMETER (IRECL_I=(IMAX+3)*4)
      PARAMETER (NX=XYRNG/XYRESO+1)                                    
      PARAMETER (NY=XYRNG/XYRESO+1)                                    
      PARAMETER (IRECL_O=NX*4)
      PARAMETER (BEAM_W=0.95)
      PARAMETER (UNDEF=-999.0)
C     
      DIMENSION    RTDAT(IMAX,JMAX,KMAX),DIS(IMAX)
      DIMENSION    SEC(JMAX),AZ(JMAX),EL(JMAX)
      DIMENSION    FDAT(IMAX+3,JMAX+1)
      DIMENSION    XYDAT(NX,NY,KMAX),DDMIN(NX,NY)
      CHARACTER*60 FIN,FOUT
      CHARACTER*64 FOUT_CTL
C                                                                      
      RAD=3.14159/180.                                                 
      ir=0
      ir11=0
C
C*** Enter file names ****
 150  FORMAT(A60)
      write(6,*)' Input_file name?  (e.g. gart.xxxx)'
      READ(5,150) FIN                                                
      write(6,*)' Output_file name? (e.g. gaxy.xxxx)'
      READ(5,150) FOUT                                                
      write(6,*)
C
C*** Open the INPUT file ****
      open(2,file=FIN,form='formatted',status='old',
     +       access='direct',recl=IRECL_I,err=999)
C
C*** Open the OUTPUT files ****
      len_fout=INDEX(FOUT,' ')-1
      FOUT_CTL=FOUT(1:len_fout)//'.ctl'
C
      open(11,file=FOUT,form='unformatted',
     +        access='direct',recl=IRECL_O,err=999)
      open(12,file=FOUT_CTL,form='formatted',err=999)
C
C--------------------------------------------------------------------- 
C*** Read data ****                              
  100 FORMAT(1000A4)
      DO 200 k=1,KMAX 
      DO 210 j=1,JMAX+1
        ir=ir+1                              
 210    READ(UNIT=2,FMT=100,REC=ir)(FDAT(i,j),i=1,IMAX+3)

      IF (k.EQ.1) then
        DO 220 i=1,IMAX
          ii=i+3
 220      DIS(i)=FDAT(ii,1)
        DO 230 j=1,JMAX
          jj=j+1
          SEC(j)=FDAT(1,jj)
          AZ(j) =FDAT(2,jj)
 230      EL(j) =FDAT(3,jj)
      ENDIF
      
      DO 250 j=1,JMAX
      DO 250 i=1,IMAX
        ii=i+3
        jj=j+1
        RTDAT(i,j,k) =FDAT(ii,jj)
 250  CONTINUE
 200  CONTINUE
      close(2)

C*** Get mean EL & max of range ****
      elmean=0.0
      elnum=0.0
      DO 600 j=1,JMAX
        IF(EL(j).GT.-2.0) THEN
          elmean=elmean+EL(j)
          elnum=elnum+1
        ENDIF
 600  CONTINUE
      elmean=elmean/elnum
      rangemax=DIS(IMAX)*0.001
      write(6,*) 'EL mean=',elmean,'  Range max=',rangemax
C
C*** Init. array ****
      DO 700 n=1,NY
      DO 700 m=1,NX
 700    DDMIN(m,n)=rangemax

      DO 710 k=1,KMAX
      DO 710 n=1,NY
      DO 710 m=1,NX
 710    XYDAT(m,n,k)=UNDEF
C
C----------------------------------------------------------------------
C*** COORDINATE CONVERSION FORM R-T TO X-Y ****                        
      DO 1000 j=1,JMAX
        IF (ABS(EL(j)-elmean) .GT. 0.5) THEN
          write(6,*) "skip ray AZ/EL=",AZ(j),EL(j)
          GOTO 1000
        ENDIF

        DO 1200 i=1,IMAX
          rr=DIS(i)*0.001
          xx=rr*sin(AZ(j)*RAD)
          yy=rr*cos(AZ(j)*RAD)

C**** Find nearest grid point ***
          mc=INT((xx+XC)/XYRESO+0.5)+1
          nc=INT((yy+YC)/XYRESO+0.5)+1
          IF (mc.LT.1 .OR. mc.GT.NX) GOTO 1200
          IF (nc.LT.1 .OR. nc.GT.NY) GOTO 1200

C*** Get weighting grid data ***
          halfbeam=rr*sin(BEAM_W*0.5*RAD)
          ngrd=INT((halfbeam+XYRESO)/XYRESO)
          m1=mc-ngrd
          m2=mc+ngrd
          IF (m1.LT.1)  m1=1
          IF (m2.GT.NX) m2=NX
          n1=nc-ngrd
          n2=nc+ngrd
          IF (n1.LT.1)  n1=1
          IF (n2.GT.NY) n2=NY

          DO 1500 n=n1,n2
          DO 1500 m=m1,m2
            gx=float(m-1)*XYRESO-XC
            gy=float(n-1)*XYRESO-YC
            range=(gx**2+gy**2)**0.5
            IF (range.GT.rangemax) GOTO 1500
            dd=((xx-gx)**2+(yy-gy)**2)**0.5
            IF (dd.GT.(halfbeam+XYRESO*0.5)) GOTO 1500
            IF (dd.LT.DDMIN(m,n)) THEN
               DDMIN(m,n)=dd
               DO 1600 k=1,KMAX
 1600            XYDAT(m,n,k)=RTDAT(i,j,k)
            ENDIF
 1500     CONTINUE

 1200   CONTINUE
 1000 CONTINUE
C                                                                      
C*** WRITE DATA ***                                            
      DO 3000 k=1,KMAX
      DO 3000 n=1,NY
        ir11=ir11+1
 3000   WRITE(11,rec=ir11) (XYDAT(m,n,k),m=1,NX)

 170  FORMAT(/' ----- OUTPUT DATA ----- ',
     +       I3,' * ',I3,' * ',I3,' params * 4 byte') 
      write(6,170) NX,NY,KMAX
C
C*** WRITE GrADS ctl file ***
 171  FORMAT('DSET      ^',A60)
 172  FORMAT('TITLE     DOW (data in XY coordinates make by dow2d.f')
 173  FORMAT('UNDEF     ',F8.2)
 174  FORMAT('XDEF       ',I3,' LINEAR  ',F10.2,'    ',F5.2)
 175  FORMAT('YDEF       ',I3,' LINEAR  ',F10.2,'    ',F5.2)
 176  FORMAT('ZDEF       ',I3,' LINEAR  ',F10.2,'    ',F5.2)
 177  FORMAT('TDEF       ',I3,' LINEAR   00:00Z01JAN2000  1mn')
 178  FORMAT('VARS     6')
 179  FORMAT('VE           1   99   Doppler velocity [m/s]')
 180  FORMAT('DM           1   99   Power in dBm [dBm]')
 181  FORMAT('NCP          1   99   Normalized Coherent Power')
 182  FORMAT('SW           1   99   Spectral width [m/s]')
 183  FORMAT('DZ           1   99   Reflectivity factor [dBz]')
 184  FORMAT('DCZ          1   99   Coherent Reflectivity factor [dBz]')
 185  FORMAT('ENDVARS')
      write(12,171) FOUT
      write(12,172)
      write(12,173) UNDEF
      write(12,174) NX,XMIN,XYRESO
      write(12,175) NY,YMIN,XYRESO
      write(12,176) 1,elmean,1.0
      write(12,177) 1
      write(12,178)
      write(12,179)
      write(12,180)
      write(12,181)
      write(12,182)
      write(12,183)
      write(12,184)
      write(12,185)

 999  CONTINUE                                                         
      close(11)
      close(12)  
      STOP                                                             
      END                                                              
