C**********************************************************************
C  ppi2d.f (is the same as spol2dn.f made on March 03,1999) 
C
C  This program is to convert R-T data (result swp2ga) 
C  into X-Y data (2D Cartesian-space PPI) using nearest point method
C  for NCAR SPOL
C
C                                  March 03, 1999   by Shinsuke Satoh
C
C  INPUT file:  R-T data in 4-byte real (result from swp2ga)
C               to read 4 parameters (DM, DZ, NCP, VR)
C  OUTPUT file: X-Y data of Ze[dBZ] & Vr[m/s] in 4-byte real
C               GrADS ctl file (xxxxxx.ctl)
C                                                                      
C  ALGORITHM: coodinate convertion using a nearest point
C             
C          
C (Parameter)
C    XYRESO    : Horizontal grid size(km)
C    XYRNG     : Output domain size (km)
C    XMIN,YMIN : First X/Y range(km) in the output domain
C    XC,YC     : Location of the radar site, Distance(km) from X/YMIN
C    BEAM_W    : Horizontal beam width(deg.)
C
C**********************************************************************
C*** Ser Parameters ****
      PARAMETER (XYRESO=0.5)
      PARAMETER (XYRNG=160.0,XMIN=-80.0,YMIN=-80.0)                    
      PARAMETER (XC=0.0-XMIN,YC=0.0-YMIN)                              
      PARAMETER (IMAX=534)
      PARAMETER (JMAX=408)
      PARAMETER (IRECL_I=(IMAX+3)*4)
      PARAMETER (NX=XYRNG/XYRESO+1)                                    
      PARAMETER (NY=XYRNG/XYRESO+1)                                    
      PARAMETER (IRECL_O=NX*4)
      PARAMETER (BEAM_W=0.94)
      PARAMETER (UNDEF=-999.0)
C     
      COMMON /DATA/ RDM(534,408),RDZ(534,408),RNCP(534,408),
     +              RVR(534,408),DIS(534),SEC(408),AZ(408),EL(408)
C                                                                      
      DIMENSION ZE(NX,NY),VR(NX,NY),DDMIN(NX,NY)
      CHARACTER*60 FIN,FOUT
      CHARACTER*64 FOUT_CTL
C                                                                      
      RAD=3.14159/180.                                                 
      ir=0
      ir11=0
C
C*** Enter file names ****
 150  FORMAT(A60)
      write(6,*)' Input_file name?'
      READ(5,150) FIN                                                
      write(6,*)' Output_file name?'
      READ(5,150) FOUT                                                
      write(6,*)
C
C*** OPEN the INPUT file ****
      open(2,file=FIN,form='formatted',status='old',
     +       access='direct',recl=IRECL_I,err=999)
C
C*** OPEN the OUTPUT file ****
      len_fout=INDEX(FOUT,' ')-1
      FOUT_CTL=FOUT(1:len_fout)//'.ctl'
C
      open(11,file=FOUT,form='unformatted',
     +        access='direct',recl=IRECL_O,err=999)
      open(12,file=FOUT_CTL,form='formatted',err=999)
C
C--------------------------------------------------------------------- 
C*** READ DATA ***                                               
      CALL READSPOL(ir)
      close(2)

      DO 600 j=1,JMAX
 600    elmean=elmean+EL(j)
      elmean=elmean/JMAX
      rangemax=DIS(IMAX)*0.001
      write(6,*) 'EL mean=',elmean,'  Range max=',rangemax
C
C*** Init. DDMIN ****
      DO 700 n=1,NY
      DO 700 m=1,NX
        DDMIN(m,n)=rangemax
        ZE(m,n)=UNDEF
 700    VR(m,n)=UNDEF
C
C----------------------------------------------------------------------
C*** COORDINATE CONVERSION FORM R-T TO X-Y ****                        
      DO 1000 j=1,JMAX
        IF (ABS(EL(j)-elmean) .GT. 0.2) THEN
          write(6,*) "skip ray AZ/EL=",AZ(j),EL(j)
          GOTO 1000
        ENDIF

        DO 1200 i=1,IMAX
          rr=DIS(i)*0.001
          xx=rr*sin(AZ(j)*RAD)
          yy=rr*cos(AZ(j)*RAD)

C**** Find nearest grid point ***
          mc=INT((xx+XC)/XYRESO+0.5)+1
          nc=INT((yy+YC)/XYRESO+0.5)+1
          IF (mc.LT.1 .OR. mc.GT.NX) GOTO 1200
          IF (nc.LT.1 .OR. nc.GT.NY) GOTO 1200

C*** Get weighting grid data ***
          halfbeam=rr*sin(BEAM_W*0.5*RAD)
          ngrd=INT((halfbeam+XYRESO)/XYRESO)
          m1=mc-ngrd
          m2=mc+ngrd
          IF (m1.LT.1)  m1=1
          IF (m2.GT.NX) m2=NX
          n1=nc-ngrd
          n2=nc+ngrd
          IF (n1.LT.1)  n1=1
          IF (n2.GT.NY) n2=NY

          DO 1500 n=n1,n2
          DO 1500 m=m1,m2
            gx=float(m-1)*XYRESO-XC
            gy=float(n-1)*XYRESO-YC
            range=(gx**2+gy**2)**0.5
            IF (range.GT.rangemax) GOTO 1500
            dd=((xx-gx)**2+(yy-gy)**2)**0.5
            IF (dd.GT.(halfbeam+XYRESO*0.5)) GOTO 1500
            IF (dd.LT.DDMIN(m,n)) THEN
               DDMIN(m,n)=dd
               IF (RDZ(i,j).GT.-10.0) THEN
                  ZE(m,n)=RDZ(i,j)
               ENDIF
               IF (RVR(i,j).GT.UNDEF) THEN
                  VR(m,n)=RVR(i,j)
               ENDIF
            ENDIF
 1500     CONTINUE

 1200   CONTINUE
 1000 CONTINUE
C                                                                      
C*** WRITE DATA ***                                            
      DO 3000 N=1,NY
        ir11=ir11+1
 3000   WRITE(11,rec=ir11) (ZE(M,N),M=1,NX)
      DO 3010 N=1,NY
        ir11=ir11+1
 3010   WRITE(11,rec=ir11) (VR(M,N),M=1,NX)

 170  FORMAT(/' ----- OUTPUT DATA ----- ',
     +       I3,' * ',I3,' * 2 param * 4 byte') 
      write(6,170) NX,NY
C
C*** WRITE GrADS ctl file ***
 171  FORMAT('DSET      ^',A60)
 172  FORMAT('TITLE     Bistatic data analysys (spol2d)')
 173  FORMAT('UNDEF     ',F8.2)
 174  FORMAT('XDEF       ',I3,' LINEAR  ',F10.2,'    ',F5.2)
 175  FORMAT('YDEF       ',I3,' LINEAR  ',F10.2,'    ',F5.2)
 176  FORMAT('ZDEF       ',I3,' LINEAR  ',F10.2,'    ',F5.2)
 177  FORMAT('TDEF       ',I3,' LINEAR   02:30Z30MAY1997  1mn')
 178  FORMAT('VARS     2')
 179  FORMAT('ze       1   99   Reflectivity factor [dBZ]')
 180  FORMAT('vr       1   99   Doppler velocity [m/s]')
 181  FORMAT('ENDVARS')
      write(12,171) FOUT
      write(12,172)
      write(12,173) UNDEF
      write(12,174) NX,XMIN,XYRESO
      write(12,175) NY,YMIN,XYRESO
      write(12,176) 1,elmean,1.0
      write(12,177) 1
      write(12,178)
      write(12,179)
      write(12,180)
      write(12,181)

C--------------------------------------------------------------------- 
 999  CONTINUE                                                         
      close(11)
      close(12)  
      STOP                                                             
      END                                                              
C**********************************************************************
      SUBROUTINE READSPOL(ir)
C     *Subroutine to read DATA, Time, Azimuth, Elevation, and Distance  
C                                       March 02, 1999   Shinsuke Satoh  
C**********************************************************************
      COMMON /DATA/ RDM(534,408),RDZ(534,408),RNCP(534,408),
     +              RVR(534,408),DIS(534),SEC(408),AZ(408),EL(408)
      DIMENSION   fdata(537,409)
C                                                                      
C*** Parameter loop ***
      do 200 k=1,4 
C
  100 FORMAT(537A4)
C*** RAW DATA READ AND DATA ATTRIBUTE ***
      do 310 j=1,409
        ir=ir+1                              
 310    READ(UNIT=2,FMT=100,REC=ir)(fdata(i,j),i=1,537)

      if (k.eq.1) then
        do 400 i=1,534
          ii=i+3
 400      DIS(i)=fdata(ii,1)
        do 410 j=1,408
          jj=j+1
          SEC(j)=fdata(1,jj)
          AZ(j) =fdata(2,jj)
 410      EL(j) =fdata(3,jj)
      endif
      
      do 500 j=1,408
      do 500 i=1,534
        ii=i+3
        jj=j+1
        if (k.eq.1) RDM(i,j) =fdata(ii,jj)
        if (k.eq.2) RDZ(i,j) =fdata(ii,jj)
        if (k.eq.3) RNCP(i,j)=fdata(ii,jj)
        if (k.eq.4) RVR(i,j) =fdata(ii,jj)
 500  continue

 200  continue
 900  continue                                                         
      RETURN                                                           
      END                                                              
