/* 	$Id: ddb_common.c,v 1.63 1997/10/16 22:16:56 oye Exp $	 */

#ifndef lint
static char vcid[] = "$Id: ddb_common.c,v 1.63 1997/10/16 22:16:56 oye Exp $";
#endif /* lint */

# define NEW_ALLOC_SCHEME
# define WEN_CHOWS_ALGORITHM

/*
 * This file contains the following routines
 * 
 * 
 * dd_absorb_cfac
 * dd_align_structs
 * dd_alloc_data_field
 * dd_azimuth_angle
 * dd_cell_num
 * dd_clear_pisp
 * dd_clip_gate
 * dd_compress
 * dd_copy_pisp
 * dd_datum_size
 * dd_delimit
 * dd_drift
 * dd_earthr
 * dd_elevation_angle
 * dd_find_desc
 * dd_find_field
 * dd_fixed_angle
 * dd_get_cfac
 * dd_get_lims
 * dd_get_time_lims
 * dd_givfld
 * dd_heading
 * dd_itsa_physical_dev
 * dd_latitude
 * dd_latlon_relative
 * dd_latlon_shift
 * 
 * dd_link_dev_name
 * dd_longitude
 * dd_malloc_radd
 * dd_nav_rotation_angle
 * dd_nav_tilt_angle
 * dd_ndx_name
 * dd_pitch
 * 
 * dor_print_asib
 * dor_print_celv
 * dor_print_parm
 * dor_print_radd
 * dor_print_ryib
 * dor_print_swib
 * dor_print_vold
 * 
 * dd_radar_angles
 * dd_radar_name
 * dd_radar_namec
 * dd_radar_selected
 * dd_range_gate
 * dd_ray_coverage
 * dd_razel_to_xyz
 * dd_reset_d_limits
 * dd_return_id_num
 * dd_return_rotang1
 * dd_roll
 * dd_rotang_seek
 * dd_rotang_table
 * dd_rotation_angle
 * dd_rng_info
 * 
 * dd_site_name
 * dd_src_fid
 * dd_tilt_angle
 * dd_time_keeper
 * dd_tokens
 * dd_uniform_cell_spacing
 * 
 * 
 * dd_whiteout
 * dd_xy_plane_horizontal
 * dgi_buf_rewind
 * fp_bin_search
 * swap2
 * swap4
 * 
 * 
 * 
 */

# include "Volume.h"
# include "Comment.h"
# include "RadarDesc.h"
# include "Correction.h"
# include "Parameter.h"
# include "CellVector.h"
# include "CellSpacing.h"
# include "Sweep.h"
# include "Platform.h"
# include "Ray.h"
# include "Pdata.h"
# include "Qdata.h"
# include "FieldParam.h"
# include "super_SWIB.h"

# include "dd_defines.h"
# include "dorade_share.h"
# include "dd_general_info.h"

# include "input_limits.h"
# include "input_sweepfiles.h"
# include "dd_stats.h"
# include <time.h>
# include <sys/time.h>
# include "point_in_space.h"

# include "dd_math.h"
# include <sys/stat.h>
# include "function_decl.h"
# include "dgi_func_decl.h"

static char *dd_sweep_modes[] =
{ "CAL", "PPI", "COP", "RHI", "VER", "TAR", "MAN", "IDL", "SUR", "AIR"
	, "???", "???", "???", "???", "???", "???" };

char *dd_align_structs();
void dd_get_cfac();
void dd_get_lims();
struct dd_general_info *dd_ini();
void dd_reset_d_limits();
char *put_tagged_string();

# define MAX_TAGS 512
static char *tags[MAX_TAGS], *tagged_strings[MAX_TAGS];
static int num_tags=0;

/* c...mark */

/* c------------------------------------------------------------------------ */

int
dd_get_scan_mode(mne)
  char *mne;
{
    int jj;
    
    for(jj=0; jj < MAX_MODES; jj++) {
	if(strncmp(mne, dd_sweep_modes[jj], 3) == 0) {
	    return(jj);
	}
    }
    return(-1);
}
/* c------------------------------------------------------------------------ */

void dd_absorb_cfac( list, radar_name, cfac)
  char *list, *radar_name;
  struct correction_d *cfac;
{
    char *a=list;
    char *dd_link_dev_name();
    char str[256];
    FILE *stream;


    if(strstr(list, "CLEAR_CORR")) {
	/* assume cfac descriptors have been set to zero. i.e. do nothing! */
	printf("Clear Corrections!\n");
	return;
    }

    if(!(a=dd_link_dev_name(radar_name, list, str))) {
	return;
    }

    printf("Opening cfac file: %s\n", str);
    if((stream = fopen(str, "r"))) {
	dd_get_cfac(stream, cfac);
	fclose(stream);
    }
    else {
	printf("Unable to open %s\n", str);
	exit(1);
    }
    return;
}
/* c------------------------------------------------------------------------ */

char *
dd_align_structs(c, n)
  char *c;
  int n;
{
    /* pass back a pointer aligned to the next 8 byte boundary */
    int i, sod=sizeof(double);

    if(n % sod)
	  n += sod -(n % sod);
    return(c+n);
}
/* c------------------------------------------------------------------------ */

double
dd_altitude(dgi)
  struct dd_general_info *dgi;
{
    double d=dgi->dds->asib->altitude_msl;
    if(dd_isnanf(d))
	  return((double)0);
    else
	  return(d + dgi->dds->cfac->pressure_alt_corr);
}
/* c------------------------------------------------------------------------ */

double
dd_altitude_agl(dgi)
  struct dd_general_info *dgi;
{
    double d=dgi->dds->asib->altitude_agl;
    if(dd_isnanf(d))
	  return((double)0);
    else
	  return(d + dgi->dds->cfac->radar_alt_corr);
}
/* c------------------------------------------------------------------------ */

void
dd_alloc_data_field(dgi, pn)
  DGI_PTR dgi;
  int pn;
{
    int mm;
    DDS_PTR dds = dgi->dds;

    mm = (dds->celv->number_cells + 2) *
	  dd_datum_size(dds->parm[pn]->binary_format);
    mm = LONGS_TO_BYTES(BYTES_TO_LONGS(mm));

    if(mm != dds->sizeof_qdat[pn]) {
# ifdef obsolete
	printf("dd_alloc_data_field: %d\n  num_cells: %d  mm: %d  %s  %d  %d\n"
	       , pn, dds->celv->number_cells, mm
	       , dts_print(d_unstamp_time(dds->dts))
	       , dds->qdat_ptrs[pn], dds->sizeof_qdat[pn]);
# endif
	if(dds->qdat_ptrs[pn]) {
	    free(dds->qdat_ptrs[pn]);
	}
	if(dds->qdat_ptrs[pn] = (char *)malloc(mm))
	      memset(dds->qdat_ptrs[pn], 0, mm);
	else {
	    printf("Unable to allocate qdat num: %d\n", pn);
	    exit(1);
	}
	dds->sizeof_qdat[pn] = mm;
# ifdef obsolete
	printf(" %d  %d\n", dds->qdat_ptrs[pn], dds->sizeof_qdat[pn]);
# endif
    }
}
/* c------------------------------------------------------------------------ */

double
dd_azimuth_angle(dgi)
  struct dd_general_info *dgi;
{
    double d_rot;

    if(dgi->dds->radd->scan_mode == AIR) {
	/* most aircraft data */
	d_rot = FMOD360(DEGREES(dgi->dds->ra->azimuth)+360.);
    }
    else if(dgi->dds->radd->radar_type == AIR_LF ||
	dgi->dds->radd->radar_type == AIR_NOSE) {
	/* this is meant to apply only to the P3 lower fuselage data */
	d_rot = dgi->dds->ryib->azimuth;
	if(dd_isnanf(d_rot))
	      d_rot = 0;
	else
	      d_rot = FMOD360(d_rot + dgi->dds->cfac->azimuth_corr
			      + dd_heading(dgi));
    }
    else {
	d_rot = dgi->dds->ryib->azimuth;
	if(dd_isnanf(d_rot))
	      d_rot = 0;
	else
	      d_rot = FMOD360(d_rot + dgi->dds->cfac->azimuth_corr);
    }
    return(d_rot);
}
/* c------------------------------------------------------------------------ */

int
dd_cell_num(dds, parameter_num, range)
  struct dds_structs *dds;
  int parameter_num;
  float range;
{
    /* find the cell corresponding to this range using the
     * uniform spacing lookup table
     */
    int ii, ndx=0, *lut, ncMax = dds->celv->number_cells -1;
    float r;

    lut = dds->uniform_cell_lut[ndx];
    ii = .5 + (range - dds->uniform_cell_zero[ndx]) *
	  dds->rcp_uniform_cell_spacing[ndx];
    if(ii < 0)
	  ii = 0;
    else if(ii >= dds->uniform_cell_count[ndx])
	  ii = dds->uniform_cell_count[ndx] -1;

    ii = *(lut +ii);		/* now look it up */
    if(ii > ncMax) {
       ii = ncMax;
    }
    return(ii);
}
/* c------------------------------------------------------------------------ */

void dd_clear_pisp(pisp)
  struct point_in_space *pisp;
{
    int nn=pisp->sizeof_struct;
    memset(pisp, 0, nn);
    pisp->sizeof_struct = nn;
    strncpy(pisp->name_struct, "PISP", 4);
}
/* c------------------------------------------------------------------------ */

int dd_clip_gate(dgi, elev, alt, lower, upper)
  DGI_PTR dgi;
  float elev;			/* assume elev in radians */
  float alt;			/* assume alt in km */
  double lower, upper;
{
    struct platform_i *asib=dgi->dds->asib;
    struct correction_d *cfac=dgi->dds->cfac;
    struct cell_d *celv=dgi->dds->celvc;
    struct radar_angles *ra=dgi->dds->ra;
    int g, lc=celv->number_cells-1, parameter_num = 0;
    float f, r;
    double alt_msl=alt*1000., d, rcp_sin;
    double dd_elevation_angle();

    elev = RADIANS(dd_elevation_angle(dgi));

    /* radar_angles should already have been called */
    if(fabs(elev*celv->dist_cells[lc]) < .01) {
	/* close enough to horizontal */
	return(lc);
    }
    rcp_sin = 1./sin(elev);

    f = (upper-alt_msl)*rcp_sin;

    if(f > celv->dist_cells[0]) {
	if(f > celv->dist_cells[lc])
	      return(lc);
	g = dd_cell_num(dgi->dds, parameter_num, f);
	return(g);
    }
    f = (lower-alt_msl)*rcp_sin;
    if(f > celv->dist_cells[0]) {
	if(f > celv->dist_cells[lc])
	      return(lc);
	g = dd_cell_num(dgi->dds, parameter_num, f);
	return(g);
    }
    return(lc);
}
/* c------------------------------------------------------------------------ */

int dd_compress( src, dst, flag, n )
  unsigned short *src, *dst, flag;
  int n;
{
    /* implement hrd compression of 16-bit values
     * and return the number of 16-bit words of compressed data
     */
    int mark, kount=0, wcount=0, data_run;
    unsigned short *ss=src, *dd=dst;
    unsigned short *rlcode, *end=src+n-1;

    if(n < 2) {
	printf("Trying to compress less than 2 values\n");
	exit(1);
    }

    for(;ss < end;) {
	/* for each run examine the first two values
	 */
	kount = 2;
	rlcode = dd++;
	if(*(ss+1) != flag || *ss != flag) { /* data run */
	    data_run = YES;
	    *dd++ = *ss++;
	    *dd++ = *ss++;
	}
	else { /* flag run */
	    data_run = NO;
	    ss += 2;
	}

	for(;ss < end;) { /* for rest of the run */
	    if(data_run) {
		if(*(ss-1) == flag && *ss == flag && kount > 2) {
		    /* break data run
		     */
		    *rlcode = SIGN16 | --kount;
		    wcount += kount+1; /* data plus code word */
		    ss--;
		    dd--;
		    break;
		}
		/* continue the data run */
		kount++;
		*dd++ = *ss++;
	    }
	    else { /* flag run */
		if(*ss != flag) { /* break flag run */
		    *rlcode = kount;
		    wcount++; /* code word only */
		    break;
		}
		ss++;
		kount++; /* continue flag run */
	    }
	}
    }
    /* now look at the last value
     */
    if(data_run) { /* just stuff it no matter what it is */
	if(ss == end) {
	    *dd++ = *ss;
	    kount++;
	}
	*rlcode = SIGN16 | kount;
	wcount += kount +1;
    }
    else if(*ss == flag) {
	*rlcode = ++kount;
	wcount++;
    }
    else { /* there is one last datum at the end of a flag run */
	if(kount == 2) {	/* special case for just two flags */
	    *rlcode = SIGN16 | 3;
	    *dd++ = flag;
	    *dd++ = flag;
	    *dd++ = *ss;
	    wcount += 4;
	}
	else {
	    *rlcode = --kount;
	    wcount++;
	    *dd++ = SIGN16 | 2;
	    *dd++ = flag;
	    *dd++ = *ss;
	    wcount += 3;
	}
    }
    *dd++ = END_OF_COMPRESSION;
    wcount++;
    return(wcount);
}
/* c------------------------------------------------------------------------ */

void dd_copy_pisp(p0, p1)
  struct point_in_space *p0, *p1;
{
    memcpy(p1, p0, p0->sizeof_struct);
}
/* c------------------------------------------------------------------------ */

int dd_datum_size(binary_format)
  int binary_format;
{
    if(binary_format == DD_8_BITS) return(1);
    if(binary_format == DD_16_BITS) return(2);
    if(binary_format == DD_24_BITS) return(3);
    if(binary_format == DD_32_BIT_FP) return(4);
    if(binary_format == DD_16_BIT_FP) return(2);
    return(0);
}
/* c------------------------------------------------------------------------ */

char *dd_delimit(c)
  char *c;
{
    /* advance to next delimiter */
    for(;*c != ' ' && *c != '\t' && *c != '\n' &&
	*c != '<' && *c != '>' && *c != '\0'&& *c != ';'; c++ );
    return(c);
}
/* c------------------------------------------------------------------------ */

double
dd_drift(dgi)
  struct dd_general_info *dgi;
{
    double d=dgi->dds->asib->drift_angle;
    if(dd_isnanf(d))
	  return((double)0);
    else
	  return(d +dgi->dds->cfac->drift_corr);
}
/* c------------------------------------------------------------------------ */

double
dd_earthr(lat)
  double lat;
{
    static double *earth_r=NULL;

    double major=6378388;	/* radius in meters */
    double minor=6356911.946;
    double tt;
    double d, theta=0, x, y;
    int ii, nn;
    
    if(!earth_r) {
	earth_r = (double *)malloc(20*sizeof(double));

	for(ii=0; theta < 90.; ii++, theta += 5.) {
	    /* create an entry every 5 degrees
	     */
	    tt = tan(RADIANS(theta));
	    d = sqrt(1.+SQ(tt*major/minor));
	    x = major/d;
	    y = x*tt;
	    *(earth_r +ii) = sqrt(SQ(x) + SQ(y));
	}
    }
    nn = fabs(lat*.2);
    d = nn < 18 ? *(earth_r +nn) : minor;
    d *= .001;			/* km.! */
    return(d);
}
/* c------------------------------------------------------------------------ */

double
dd_elevation_angle(dgi)
  struct dd_general_info *dgi;
{
    int mark;
    double d, d_rot;
    double AzmR, ElR, PitchR, RollR, z;

    if(dgi->dds->radd->scan_mode == AIR) {
	/* most aircraft data */
	d_rot = DEGREES(dgi->dds->ra->elevation);
    }
    else if(dgi->dds->radd->radar_type == AIR_LF) {
	/* this is meant to apply only to the P3 lower fuselage data.
	 * the elevation angle is recorded relative to the aircraft
	 * but the antenna is trying to maintain a constant elevation
	 * relative to the earth
	 *
	 * This code courtesy of Bob Hueftle MRD/NOAA
	 */
	d = dgi->dds->ryib->elevation;

	if(dd_isnanf(d)) {
	    d_rot = 0;
	}
	else {
	    ElR = RADIANS(dgi->dds->ryib->elevation
			  + dgi->dds->cfac->elevation_corr);

	    AzmR = RADIANS(dgi->dds->ryib->azimuth
		+ dgi->dds->cfac->azimuth_corr);
	    PitchR = RADIANS(dd_pitch(dgi));
	    RollR = RADIANS(dd_roll(dgi));
	    
	    z = cos(AzmR)*cos(ElR)*sin(PitchR)
		  + sin(ElR)*cos(PitchR)*cos(RollR)
			- sin(AzmR)*cos(ElR)*cos(PitchR)*sin(RollR);
	    
	    if(z > 1.) z = 1.;
	    else if(z < -1.) z = -1.;
	    d_rot = DEGREES(asin(z));
	}
    }
    else {
	switch(dgi->dds->radd->scan_mode) {
	    
	case TAR:
	case VER:
	default:
	    d = dgi->dds->ryib->elevation;
	    
	    if(dd_isnanf(d)) {
		d_rot = 0;
	    }
	    else {
		d_rot = d + dgi->dds->cfac->elevation_corr;
	    }
	    break;
	}
    }
    return(d_rot);
}
/* c------------------------------------------------------------------------ */

char *dd_find_desc(a, b, desc)
  char *a, *b, *desc;
{
    struct generic_descriptor *gd;
    while(a < b) {
	if(strncmp(a, desc, 4) == 0)
	      return(a);
	gd = (struct generic_descriptor *)a;
	a += gd->sizeof_struct;
    }
    return(0);
}
/* c------------------------------------------------------------------------ */

int
dd_find_field(dgi, name)
  struct dd_general_info *dgi;
  char *name;
{
    int ii=0, nc;
    struct dds_structs *dds=dgi->dds;

    if((nc = strlen(name)) < 1)
	  return(-1);

    for(ii=0; ii < MAX_PARMS; ii++) {
	if(!dds->field_present[ii])
	      continue;
	if(!strncmp(name, dds->parm[ii]->parameter_name, nc)) {
	    /* destination field! */
	    return(ii);
	}
    }
    return(-1);
}
/* c------------------------------------------------------------------------ */

void dd_get_cfac(stream, cfac)
  FILE *stream;
  struct correction_d *cfac;
{
    /* routine to absorb an ascii version of the cfac info and
     * stuff it into the cfac struct
     */
    int nn, nt;
    char str[256], *aa, *bb;
    char string_space[256], *strptrs[16];
    double d, atof();
    float f;

    for(d = 0; aa = fgets(string_space, 256, stream); d = 0) { /* next line */
	if(*aa == '#' || *aa == '!') /* comment */
	      continue;
	strcpy(string_space, aa);
	if((nt = dd_tokenz(string_space, strptrs, " =\t\n")) < 2) {
	    continue;
	}
	if((nn = sscanf(strptrs[1], "%f", &f)) == 1) {
	    d = f;
	}
	bb = strptrs[0];

	if(strstr(bb, "azimuth_corr")) {
	    cfac->azimuth_corr = d;
	}
	else if(strstr(bb, "elevation_corr")) {
	    cfac->elevation_corr = d;
	}
	else if(strstr(bb, "range_delay_corr")) {
	    cfac->range_delay_corr = d;
	}
	else if(strstr(bb, "longitude_corr")) {
	    cfac->longitude_corr = d;
	}
	else if(strstr(bb, "latitude_corr")) {
	    cfac->latitude_corr = d;
	}
	else if(strstr(bb, "pressure_alt_corr")) {
	    cfac->pressure_alt_corr = d;
	}
	else if(strstr(bb, "radar_alt_corr")) {
	    cfac->radar_alt_corr = d;
	}
	else if(strstr(bb, "ew_gndspd_corr")) {
	    cfac->ew_gndspd_corr = d;
	}
	else if(strstr(bb, "ns_gndspd_corr")) {
	    cfac->ns_gndspd_corr = d;
	}
	else if(strstr(bb, "vert_vel_corr")) {
	    cfac->vert_vel_corr = d;
	}
	else if(strstr(bb, "heading_corr")) {
	    cfac->heading_corr = d;
	}
	else if(strstr(bb, "roll_corr")) {
	    cfac->roll_corr = d;
	}
	else if(strstr(bb, "pitch_corr")) {
	    cfac->pitch_corr = d;
	}
	else if(strstr(bb, "drift_corr")) {
	    cfac->drift_corr = d;
	}
	else if(strstr(bb, "rot_angle_corr")) {
	    cfac->rot_angle_corr = d;
	}
	else if(strstr(bb, "tilt_corr")) {
	    cfac->tilt_corr = d;
	}
	else {
	    printf("%s\n**** Not a usable correction factor ****\n", aa);
	}
    }
}
/* c------------------------------------------------------------------------ */

void dd_get_lims(str_ptrs, ii, nt, dlims)
  char *str_ptrs[];
  int ii, nt;
  struct d_limits **dlims;
{
    /* extract the next set of limits
     * either two numbers seperated by a string like "<" or "-" or
     * a single number
     */

    int kk;
    float f, f1=-MAX_FLOAT, f2=MAX_FLOAT;

    dd_reset_d_limits(dlims);

    if(ii < nt) {
	sscanf(str_ptrs[ii], "%f", &f1);
    }
    kk = nt == 2 ? 1 : ii+2;
    if(kk < nt) {
	sscanf(str_ptrs[kk], "%f", &f2);
    }

    (*dlims)->lower = f1;
    (*dlims)->upper = f2;
    return;
}
/* c------------------------------------------------------------------------ */

int dd_givfld( dgi, ndx, g1, n, dd, badval )
  DGI_PTR dgi;
  int *ndx, *g1, *n;
  float *dd, *badval;
{
    /*
     * dgi: dd_general_info struct
     * ndx: the index of the desired field
     * g1:  the gate number of where to start (1 => the first gate)
     * n:   the number of gates desired
     * dd:  the destination array for the floating point values
     * bad_val: what the bad data flag looks like in floating point
     */
    DDS_PTR dds=dgi->dds;
    static int err_count=0;
    unsigned char *a;
    char *buf, *deep6=0;
    float bias;
    short gate[MAXCVGATES];
    int bad, i, idx= *ndx, m, df, dd_hrd16(), mark;
    int gndx= *g1-1, ng=dgi->dds->celv->number_cells;
    float rcp_scale, *ff;
    short *ss;
    
    if(idx < 0 || idx >= dgi->num_parms) {
	if(++err_count > 11) {
	    i = *deep6;
	}
	return((int)0);
    }
    bias = dds->parm[idx]->parameter_bias;
# ifdef NEW_ALLOC_SCHEME
    buf = dds->qdat_ptrs[idx];
# else
    buf = (char *)dds->rdat[idx] + sizeof(struct paramdata_d);
# endif
    rcp_scale = 1./dds->parm[idx]->parameter_scale;
    *badval = DD_UNSCALE((float)dds->parm[idx]->bad_data, rcp_scale, bias);
    bad = dds->parm[idx]->bad_data;
    ng = gndx+(*n) < ng ? *n : ng-gndx;

    df = dds->parm[idx]->binary_format;

    if( df == DD_8_BITS ) { 
	a = (unsigned char *)buf;
	for(a+=gndx,i=0; i < ng; i++,a++,dd++){
	    if(*a == bad)
		  *dd = *badval;
	    else
		  *dd = DD_UNSCALE((float)(*a), rcp_scale, bias);
	}
    }
    else if( df == DD_16_BITS ) { 
	ss = (short *)buf;
	for(ss+=gndx,i=0; i < ng; i++,ss++,dd++){
	    if(*ss == bad)
		  *dd = *badval;
	    else
		  *dd = DD_UNSCALE((float)(*ss), rcp_scale, bias);
	}
    }
    else if( df == DD_24_BITS ) { 
    }
    else if( df == DD_32_BIT_FP ) { 
	ff = (float *)buf;
	ff += gndx;
	memcpy((char*)dd, (char *)ff, ng*sizeof(float));
    }
    return(ng);
}
/* c------------------------------------------------------------------------ */

double dd_heading(dgi)
  struct dd_general_info *dgi;
{
    double d=dgi->dds->asib->heading;
    if(dd_isnanf(d))
	  return((double)0);
    else
	  return(d +dgi->dds->cfac->heading_corr);
}

/* c------------------------------------------------------------------------ */

int dd_itsa_physical_dev(s)
  char *s;
{
    /* check the string to see if it contains a physical device name */
    if(strstr(s, "/dev/"))
	  return(YES);
    return(NO);
}
/* c------------------------------------------------------------------------ */

double
dd_latitude(dgi)
  struct dd_general_info *dgi;
{
    double d=dgi->dds->asib->latitude;
    if(dd_isnanf(d))
	  return((double)0);
    else
	  return(d + dgi->dds->cfac->latitude_corr);
}
/* c------------------------------------------------------------------------ */

void dd_latlon_relative(p0, p1)
  struct point_in_space *p0, *p1;
{
    /* routine to calculate (x,y,z) for p1 so as to line up
     * with (0,0,0) for p0
     */
    double del_lat, del_lon, lat=RADIANS(p1->latitude);
    double dd_earthr(), r_earth;

    del_lat = RADIANS(p0->latitude) - lat;
    del_lon = RADIANS(p0->longitude) - RADIANS(p1->longitude);

    p1->x = (r_earth = dd_earthr(p0->latitude)) * cos(lat) * sin(del_lon);
    p1->y = r_earth * sin(del_lat);

    p1->z = p0->altitude + p0->z - p1->altitude;
}
/* c------------------------------------------------------------------------ */

void dd_latlon_shift(p0, p1)
  struct point_in_space *p0, *p1;
{
    /* routine to calculate lat/lon/alt for p1 so as to line up
     * with (x,y,z) for p0
     *
     * might want to investigate the formula:
     * c = 2 * R * sin(theta/2) where c is the cord length
     * of the arc (R * theta). Perhaps it will take into account
     * the earths curvature.
     */
    double d, r;
    double dd_earthr(), r_earth;

    r_earth = dd_earthr(p0->latitude);
    r = r_earth + p0->altitude + p0->z;
    p1->altitude = r -r_earth;

    p1->latitude = RADIANS(p0->latitude) + asin(p0->y/r);
    p1->longitude = RADIANS(p0->longitude)
	  + asin(p0->x/(r * cos(p1->latitude)));

    p1->latitude = DEGREES(p1->latitude);
    p1->longitude = DEGREES(p1->longitude);
}
/* c------------------------------------------------------------------------ */

char *dd_link_dev_name(radar_name, dev_links, str)
  char *radar_name, *dev_links, *str;
{
    /* this routine is looking for pairs of the form
     * "TA > /dev/nrst0 TF > /dev/nrst1"
     * it want to return a pointer the the device name
     * to be associated with the radar
     */
    char *str_ptrs[32], string_space[256];
    char radar[16], *str_terminate();
    int ii, nt;

    str_terminate(radar, radar_name, 8);
    strcpy(string_space, dev_links);
    nt = dd_tokens(string_space, str_ptrs);

    for(ii=0; ii < nt; ii += 3) {
	if(strstr(radar, str_ptrs[ii])) {
	    if(ii+2 < nt) {
		strcpy(str, str_ptrs[ii+2]);
		return(str);
	    }
	}
    }
    return(NULL);
}
/* c------------------------------------------------------------------------ */

double
dd_longitude(dgi)
  struct dd_general_info *dgi;
{
    double d=dgi->dds->asib->longitude;
    if(dd_isnanf(d))
	  return((double)0);
    else
	  return(d + dgi->dds->cfac->longitude_corr);
}
/* c------------------------------------------------------------------------ */

struct radar_d *
dd_malloc_radd(dgi, config_name)
  DGI_PTR dgi;
  char *config_name;
{
    /* return a pointer to the radar descriptor with the
     * corresponding config_name or establish a new one
     * with this config name
     */
    struct radar_d *radd=NULL, **radd_ptr=dgi->dds->radd_list;
    DDS_PTR dds=dgi->dds;
    int ii, mm, nn, found=NO;
    char *aa;

    /* search for a match
     */
    if(strlen(config_name)) {
	for(mm=dds->radar_desc_count; mm-- ; radd_ptr++) {
	    if(!strncmp((*radd_ptr)->config_name, config_name, 8)) {
		radd = *radd_ptr;
		break;
	    }
	}
    }
    if(!radd) {			/* need to add a new radar descriptor */
	if(dds->radar_desc_count+1 > dds->max_radar_desc) {
	    /* need to expand the list of pointers */
	    dds->max_radar_desc += 8;
	    if(dds->radar_desc_count) {
		dds->radd_list = (struct radar_d **)
		      realloc(dds->radd_list, dds->max_radar_desc *
			      sizeof(struct radar_d *));
	    }
	    else {
		dds->radd_list = (struct radar_d **)
		      malloc(dds->max_radar_desc * sizeof(struct radar_d *));
		memset(dds->radd_list, 0, dds->max_radar_desc *
		       sizeof(struct radar_d *));
	    }
	}
	if(radd = (struct radar_d *)malloc(sizeof(struct radar_d)))
	      memset(radd, 0, sizeof(struct radar_d));
	else {
	    printf("Unable to allocate the next radar descriptor\n");
	    exit(1);
	}
	if(dds->radar_desc_count) {
	    /* copy first one to this one */
	    memcpy(radd, *dds->radd_list, sizeof(struct radar_d *));
	}
	else {			/* first one */
	    strncpy(radd->radar_des, "RADD", 4 );
	    strncpy(radd->config_name, "NONE    ", 8);
	    radd->radar_des_length = sizeof(struct radar_d);
	}
	*(dds->radd_list + dds->radar_desc_count++) = radd;
    }
    return(radd);
}
/* c------------------------------------------------------------------------ */

double
dd_nav_rotation_angle(dgi)
  struct dd_general_info *dgi;
{
    double d_rot, dd_azimuth_angle();

    switch(dgi->dds->radd->scan_mode) {

    case AIR:
	d_rot = FMOD360(dgi->dds->asib->rotation_angle +
			dgi->dds->cfac->rot_angle_corr );
	break;

    case RHI:
	d_rot = FMOD360((450.-(dgi->dds->ryib->elevation
			       + dgi->dds->cfac->elevation_corr)));
	break;

    case TAR:
	d_rot = dgi->dds->radd->radar_type != GROUND ?
	      FMOD360(dgi->dds->asib->rotation_angle +
			dgi->dds->cfac->rot_angle_corr) :
			      dd_azimuth_angle(dgi);
	break;

    default:
	d_rot = dd_azimuth_angle(dgi);
	break;
    }
    return(d_rot);
}
/* c------------------------------------------------------------------------ */

double
dd_nav_tilt_angle(dgi)
  struct dd_general_info *dgi;
{
    double d_rot;

    switch(dgi->dds->radd->scan_mode) {
	
    case AIR:
	d_rot = dgi->dds->asib->tilt + dgi->dds->cfac->tilt_corr;
	break;

    case RHI:
	d_rot = CART_ANGLE
	      (dgi->dds->ryib->azimuth +dgi->dds->cfac->azimuth_corr);
	break;

    default:
	d_rot = dgi->dds->ryib->elevation +dgi->dds->cfac->elevation_corr;
	break;
    }
    return(d_rot);
}
/* c------------------------------------------------------------------------ */

int dd_ndx_name(dgi, name2)
  DGI_PTR dgi;
  char *name2;
{
    DDS_PTR dds=dgi->dds;
    int ii, n=strlen(name2);

    if(n < 1) {
	return(-1);
    }
    ii = dd_find_field(dgi, name2);
    return(ii);
}
/* c------------------------------------------------------------------------ */

double
dd_pitch(dgi)
  struct dd_general_info *dgi;
{
    double d=dgi->dds->asib->pitch;
    if(dd_isnanf(d))
	  return((double)0);
    else
	  return(d +dgi->dds->cfac->pitch_corr);
}
/* c------------------------------------------------------------------------ */

void
dor_print_asib(asib, slm)
  struct platform_i *asib;
  struct solo_list_mgmt *slm;
{
    char *aa, bstr[128];

    aa = bstr;
    /* routine to print the contents of the descriptor */

    solo_add_list_entry(slm, " ", 1);

    sprintf(aa, "Contents of the platform descriptor  len: %d"
	    , sizeof(struct platform_i));

    /* c...mark */

    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " platform_info[4]       %s", asib->platform_info);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " platform_info_lengtth  %d", asib->platform_info_length);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " longitude              %f", asib->longitude);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " latitude               %f", asib->latitude);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " altitude_msl           %f", asib->altitude_msl);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " altitude_agl           %f", asib->altitude_agl);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " ew_velocity            %f", asib->ew_velocity);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " ns_velocity            %f", asib->ns_velocity);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " vert_velocity          %f", asib->vert_velocity);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " heading                %f", asib->heading);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " roll                   %f", asib->roll);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " pitch                  %f", asib->pitch);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " drift_angle            %f", asib->drift_angle);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " rotation_angle         %f", asib->rotation_angle);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " tilt                   %f", asib->tilt);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " ew_horiz_wind          %f", asib->ew_horiz_wind);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " ns_horiz_wind          %f", asib->ns_horiz_wind);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " vert_wind              %f", asib->vert_wind);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " heading_change         %f", asib->heading_change);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " pitch_change           %f", asib->pitch_change);
    solo_add_list_entry(slm, aa, strlen(aa));

    return;
}
/* c------------------------------------------------------------------------ */

void
dor_print_celv(celv, slm)
  struct cell_d *celv;
  struct solo_list_mgmt *slm;
{
    int ii;
    double d, delta1=1.e6, delta2;
    char *aa, bstr[128];

    aa = bstr;

    solo_add_list_entry(slm, " ", 1);

    sprintf(aa, "Contents of the cell vector descriptor  len: %d"
	    , sizeof(struct cell_d));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, "cell_spacing_des	%s", celv->cell_spacing_des);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "cell_des_len	%d", celv->cell_des_len);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "number_cells	%d", celv->number_cells);   
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "dist_cells[  0]     %f", celv->dist_cells[0]);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "dist_cells[  1]     %f", celv->dist_cells[1]);
    solo_add_list_entry(slm, aa, strlen(aa));
    
    delta1 = fabs(celv->dist_cells[  1] -celv->dist_cells[0]);

    for(ii=1; ii < celv->number_cells-1; ii++) {
	delta2 = fabs(celv->dist_cells[ii+1] -celv->dist_cells[ii]);
	if(fabs(delta2 -delta1) > 10) {
	    sprintf(aa, "dist_cells[%3d]     %f"
		    , ii, celv->dist_cells[ii]);
	    solo_add_list_entry(slm, aa, strlen(aa));
	    sprintf(aa, "dist_cells[%3d]     %f"
		    , ii+1, celv->dist_cells[ii+1]);
	    solo_add_list_entry(slm, aa, strlen(aa));
	}
	delta1 = delta2;
    }
    sprintf(aa, "dist_cells[%3d]     %f", ii, celv->dist_cells[ii]);
    solo_add_list_entry(slm, aa, strlen(aa));
    return;
}
/* c------------------------------------------------------------------------ */

void
dor_print_cfac(cfac, slm)
  struct correction_d *cfac;
  struct solo_list_mgmt *slm;
{
    char str[64], *str_terminate();
    char *aa, bstr[128];

    /* routine to print the contents of the descriptor */
    aa = bstr;
    solo_add_list_entry(slm, " ", 1);

    sprintf(aa, "Contents of the correction descriptor  len: %d\n"
	    , sizeof(struct correction_d));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, "correction_des %s       ", cfac->correction_des       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "correction_des_length %d", cfac->correction_des_length);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "azimuth_corr %f         ", cfac->azimuth_corr         );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "elevation_corr %f       ", cfac->elevation_corr       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "range_delay_corr %f     ", cfac->range_delay_corr     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "longitude_corr %f       ", cfac->longitude_corr       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "latitude_corr %f        ", cfac->latitude_corr        );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "pressure_alt_corr %f    ", cfac->pressure_alt_corr    );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "radar_alt_corr %f       ", cfac->radar_alt_corr       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "ew_gndspd_corr %f       ", cfac->ew_gndspd_corr       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "ns_gndspd_corr %f       ", cfac->ns_gndspd_corr       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "vert_vel_corr %f        ", cfac->vert_vel_corr        );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "heading_corr %f         ", cfac->heading_corr         );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "roll_corr %f            ", cfac->roll_corr            );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "pitch_corr %f           ", cfac->pitch_corr           );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "drift_corr %f           ", cfac->drift_corr           );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "rot_angle_corr %f       ", cfac->rot_angle_corr       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "tilt_corr %f            ", cfac->tilt_corr            );
    solo_add_list_entry(slm, aa, strlen(aa));

    return;
}
/* c------------------------------------------------------------------------ */

void
dor_print_parm(parm, slm)
  struct parameter_d *parm;
struct solo_list_mgmt *slm;
{
    char str[64], *str_terminate();
    char *aa, bstr[128];

    aa = bstr;


    /* routine to print the contents of the parameter descriptor */

    solo_add_list_entry(slm, " ", 1);
    sprintf(aa, "Contents of the parameter descriptor  len: %d"
	    , sizeof(struct parameter_d));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, "parameter_des[4]        %s", parm->parameter_des       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "parameter_des_length    %d", parm->parameter_des_length);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "parameter_name[8]       %s"			     
	    , str_terminate(str, parm->parameter_name			     
			    , sizeof(parm->parameter_name))                  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "param_description[40]   %s"			     
	    , str_terminate(str, parm->param_description		     
			    , sizeof(parm->param_description))      	     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "param_units[8]          %s"			     
	    , str_terminate(str, parm->param_units			     
			    , sizeof(parm->param_units))                     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "interpulse_time         %d", parm->interpulse_time     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "xmitted_freq            %d", parm->xmitted_freq        );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "recvr_bandwidth         %f", parm->recvr_bandwidth     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "pulse_width             %d", parm->pulse_width         );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "polarization            %d", parm->polarization        );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "num_samples             %d", parm->num_samples         );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "binary_format           %d", parm->binary_format       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "threshold_field         %s"			     
	    , str_terminate(str, parm->threshold_field			     
			    , sizeof(parm->threshold_field))         	     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "threshold_value         %f", parm->threshold_value     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "parameter_scale         %f", parm->parameter_scale     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "parameter_bias          %f", parm->parameter_bias      );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "bad_data                %d", parm->bad_data            );
    solo_add_list_entry(slm, aa, strlen(aa));

    return;
}
/* c------------------------------------------------------------------------ */

void
dor_print_radd(radd, slm)
  struct radar_d *radd;
  struct solo_list_mgmt *slm;
{
    char str[128], *str_terminate();
    char *aa, bstr[128];

    aa = bstr;
    /* routine to print the contents of the descriptor */
    
    solo_add_list_entry(slm, " ", 1);
    sprintf(aa, "Contents of the radar descriptor  len: %d"
	    , sizeof(struct radar_d));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, " radar_des[4]       %s", radd->radar_des        );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " radar_des_length   %d", radd->radar_des_length );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " radar_name[8]      %s"
	    , str_terminate(str, radd->radar_name, sizeof(radd->radar_name)));
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " radar_const        %f", radd->radar_const      );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " peak_power         %f", radd->peak_power       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " noise_power        %f", radd->noise_power      );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " receiver_gain      %f", radd->receiver_gain    );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " antenna_gain       %f", radd->antenna_gain     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " system_gain        %f", radd->system_gain      );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " horz_beam_width    %f", radd->horz_beam_width  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " vert_beam_width    %f", radd->vert_beam_width  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " radar_type         %d", radd->radar_type       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " scan_mode          %d", radd->scan_mode        );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " req_rotat_vel      %f", radd->req_rotat_vel    );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " scan_mode_pram0    %f", radd->scan_mode_pram0  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " scan_mode_pram1    %f", radd->scan_mode_pram1  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " num_parameter_des  %d", radd->num_parameter_des);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " total_num_des      %d", radd->total_num_des    );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " data_compress      %d", radd->data_compress    );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " data_reduction     %d", radd->data_reduction   );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " data_red_parm0     %f", radd->data_red_parm0   );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " data_red_parm1     %f", radd->data_red_parm1   );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " radar_longitude    %f", radd->radar_longitude  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " radar_latitude     %f", radd->radar_latitude   );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " radar_altitude     %f", radd->radar_altitude   );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " eff_unamb_vel      %f", radd->eff_unamb_vel    );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " eff_unamb_range    %f", radd->eff_unamb_range  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " num_freq_trans     %d", radd->num_freq_trans   );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " num_ipps_trans     %d", radd->num_ipps_trans   );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " freq1              %f", radd->freq1            );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " freq2              %f", radd->freq2            );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " freq3              %f", radd->freq3            );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " freq4              %f", radd->freq4            );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " freq5              %f", radd->freq5            );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " interpulse_per1    %f", radd->interpulse_per1  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " interpulse_per2    %f", radd->interpulse_per2  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " interpulse_per3    %f", radd->interpulse_per3  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " interpulse_per4    %f", radd->interpulse_per4  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " interpulse_per5    %f", radd->interpulse_per5  );
    solo_add_list_entry(slm, aa, strlen(aa));

    return;
}
/* c------------------------------------------------------------------------ */

void
dor_print_ryib(ryib, slm)
  struct ray_i *ryib;
  struct solo_list_mgmt *slm;
{
    char *aa, bstr[128];

    aa = bstr;

    /* routine to print the contents of the descriptor */

    solo_add_list_entry(slm, " ", 1);
    sprintf(aa, "Contents of the ray descriptor  len: %d"
	    , sizeof(struct ray_i));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, " ray_info[4]      %s", ryib->ray_info        );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " ray_info_length  %d", ryib->ray_info_length );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " sweep_num        %d", ryib->sweep_num       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " julian_day       %d", ryib->julian_day      );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " hour             %d", ryib->hour            );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " minute           %d", ryib->minute          );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " second           %d", ryib->second          );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " millisecond      %d", ryib->millisecond     );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " azimuth          %f", ryib->azimuth         );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " elevation        %f", ryib->elevation       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " peak_power       %f", ryib->peak_power      );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " true_scan_rate   %f", ryib->true_scan_rate  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " ray_status       %d", ryib->ray_status      );
    solo_add_list_entry(slm, aa, strlen(aa));

    return;
}
/* c------------------------------------------------------------------------ */

void
dor_print_swib(swib, slm)
  struct sweepinfo_d *swib;
  struct solo_list_mgmt *slm;
{
    char *str_terminate();
    char *aa, bstr[128];

    aa = bstr;

    solo_add_list_entry(slm, " ", 1);
    sprintf(aa, "Contents of the sweep descriptor  len: %d"
	    , sizeof(struct sweepinfo_d));

    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "sweep_des[4]     %s", swib->sweep_des);       
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "sweep_des_length %d", swib->sweep_des_length);
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "radar_name[8]    %s", swib->radar_name);      
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "sweep_num        %d", swib->sweep_num);       
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "num_rays         %d", swib->num_rays);        
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "start_angle      %f", swib->start_angle);     
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "stop_angle       %f", swib->stop_angle);      
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "fixed_angle      %f", swib->fixed_angle);     
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, "filter_flag      %d", swib->filter_flag);     
    solo_add_list_entry(slm, aa, strlen(aa));

    return;
}
/* c------------------------------------------------------------------------ */

void
dor_print_sswb(sswb, slm)
  struct super_SWIB *sswb;
  struct solo_list_mgmt *slm;
{
    int ii;
    char str[128], *str_terminate();
    char *aa, bstr[128];
    DD_TIME dts;
    double dt;
    time_t ttime;

    aa = bstr;

    /* routine to print the contents of the volume descriptor */

    solo_add_list_entry(slm, " ", 1);
    sprintf(aa, "Contents of the super sweepinfo descriptor  len: %d\n"
	    , sizeof(struct super_SWIB));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, " name_struct        %s   ", sswb->name_struct       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " sizeof_struct      %d   ", sswb->sizeof_struct     );
    solo_add_list_entry(slm, aa, strlen(aa));

    dts.time_stamp = sswb->last_used;
    sprintf(aa, " last_used          %d  %s ", sswb->last_used
	, dts_print( d_unstamp_time( &dts )));
    solo_add_list_entry(slm, aa, strlen(aa));

    dts.time_stamp = sswb->start_time;
    sprintf(aa, " start_time         %d  %s ", sswb->start_time
        , dts_print( d_unstamp_time( &dts )));
    solo_add_list_entry(slm, aa, strlen(aa));

    dts.time_stamp = sswb->stop_time;
    sprintf(aa, " stop_time          %d  %s ", sswb->stop_time
        , dts_print( d_unstamp_time( &dts )));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, " sizeof_file        %d   ", sswb->sizeof_file       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " compression_flag   %d   ", sswb->compression_flag  );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " volume_time_stamp  %d   ", sswb->volume_time_stamp );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " num_params         %d   ", sswb->num_params        );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " radar_name         %s   "					   
	    , str_terminate(str, sswb->radar_name, sizeof(sswb->radar_name)));
    solo_add_list_entry(slm, aa, strlen(aa));

    dts.time_stamp = sswb->d_start_time;
    sprintf(aa, " d_start_time       %.3f  %s ", sswb->d_start_time
        , dts_print( d_unstamp_time( &dts )));
    solo_add_list_entry(slm, aa, strlen(aa));

    dts.time_stamp = sswb->d_stop_time;
    sprintf(aa, " d_stop_time        %.3f  %s ", sswb->d_stop_time
        , dts_print( d_unstamp_time( &dts )));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, " version_num        %d   ", sswb->version_num       );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " num_key_tables     %d   ", sswb->num_key_tables    );
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " status             %d   ", sswb->status            );
    solo_add_list_entry(slm, aa, strlen(aa));

    for(ii=0; ii < 7; ii++) {
	sprintf(aa, "place_holder[%d] %d\n", ii, sswb->place_holder[ii]);
	solo_add_list_entry(slm, aa, strlen(aa));
    }

    for(ii=0; ii < sswb->num_key_tables; ii++) {
	sprintf(aa, "key_table[%d]->  offset: %d  size: %d  type: %d\n"
		, ii
		, sswb->key_table[ii].offset
		, sswb->key_table[ii].size
		, sswb->key_table[ii].type
		);
	solo_add_list_entry(slm, aa, strlen(aa));
    }

    return;
}
/* c------------------------------------------------------------------------ */

void
dor_print_vold(vold, slm)
  struct volume_d *vold;
  struct solo_list_mgmt *slm;
{
    char str[128], *str_terminate();
    char *aa, bstr[128];

    aa = bstr;

    /* routine to print the contents of the volume descriptor */

    solo_add_list_entry(slm, " ", 1);
    sprintf(aa, "Contents of the volume descriptor  len: %d"
	    , sizeof(struct volume_d));
    solo_add_list_entry(slm, aa, strlen(aa));

    sprintf(aa, " volume_des[4]      %s", vold->volume_des);                                                     
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " volume_des_length  %d", vold->volume_des_length);                                              
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " format_version     %d", vold->format_version     );                                            
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " volume_num         %d", vold->volume_num           );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " maximum_bytes      %d", vold->maximum_bytes        );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " proj_name[20]      %s"
	    , str_terminate(str, vold->proj_name, sizeof(vold->proj_name)));         
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " year               %d", vold->year                 );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " month              %d", vold->month                );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " day                %d", vold->day                  );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " data_set_hour      %d", vold->data_set_hour        );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " data_set_minute    %d", vold->data_set_minute      );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " data_set_second    %d", vold->data_set_second      );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " flight_num[8]      %s"
	    , str_terminate(str, vold->flight_num, sizeof(vold->flight_num)));       
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " gen_facility[8]    %s",
	    str_terminate(str, vold->gen_facility, sizeof(vold->gen_facility)));   
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " gen_year           %d", vold->gen_year             );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " gen_month          %d", vold->gen_month            );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " gen_day            %d", vold->gen_day              );                                          
    solo_add_list_entry(slm, aa, strlen(aa));
    sprintf(aa, " number_sensor_des  %d", vold->number_sensor_des    );                                          
    solo_add_list_entry(slm, aa, strlen(aa));

    return;
}
/* c------------------------------------------------------------------------ */

void dd_radar_angles( asib, cfac, ra, dgi )
  struct platform_i *asib;
  struct correction_d *cfac;
  struct radar_angles *ra;
  struct dd_general_info *dgi;
{
    /* compute the true azimuth, elevation, etc. from platform
     * parameters using Testud's equations with their different
     * definitions of rotation angle, etc.
     */
    double R, H, P, D, T, theta_a, tau_a, theta_t, tau_t, lambda_t, phi_t;
    double sinP, cosP, sinT, cosT, sinD, cosD;
    double sin_theta_rc, cos_theta_rc, sin_tau_a, cos_tau_a;
    double xsubt, ysubt, zsubt, lambda, phi;

    double d, bigR, az;
    double sin_el, cos_az, cos_el, cos_psi, cos_phi;
    double sin_pitch, cos_pitch, cos_drift, sin_tilt, cos_tilt;
    double rtilt, rtrack, rphi;
    double rpitch=RADIANS(asib->pitch +cfac->pitch_corr);
    double rhead=RADIANS(asib->heading +cfac->heading_corr);
    double rdrift=RADIANS(asib->drift_angle +cfac->drift_corr);
    double azr, azd;


# ifdef WEN_CHOWS_ALGORITHM
    /*
     * see Wen-Chau Lee's paper
     * "Mapping of the Airborne Doppler Radar Data"
     */
    d = asib->roll;
    R = dd_isnanf(d) ? 0 : RADIANS(d +cfac->roll_corr);

    d = asib->pitch;
    P = dd_isnanf(d) ? 0 : RADIANS(d +cfac->pitch_corr);

    d = asib->heading;
    H = dd_isnanf(d) ? 0 : RADIANS(d +cfac->heading_corr);

    d = asib->drift_angle;
    D = dd_isnanf(d) ? 0 : RADIANS(d +cfac->drift_corr);

    
    if( dgi->dds->radd->radar_type == AIR_LF ||
       dgi->dds->radd->radar_type == AIR_NOSE )
      {
	 /* tilt is the azimuth here (this is not in the paper) */
	 d = 90. - FABS( dgi->dds->ryib->azimuth );
	 tau_a = dd_isnanf(d) ? 0 : RADIANS(d +cfac->azimuth_corr);

	 /* and the rotation angle is the tilt */
	 d = asib->tilt;
	 theta_a = dd_isnanf(d) ? 0 : RADIANS(d +cfac->tilt_corr);
      }
    else
      {
	 d = asib->rotation_angle;
	 theta_a = dd_isnanf(d) ? 0 : RADIANS(d +cfac->rot_angle_corr);

	 d = asib->tilt;
	 tau_a = dd_isnanf(d) ? 0 : RADIANS(d +cfac->tilt_corr);
      }
    T = H + D;
    sinP = sin(P);
    cosP = cos(P);
    sinT = sin(T);
    cosT = cos(T);
    sinD = sin(D);
    cosD = cos(D);
    sin_tau_a = sin(tau_a);
    cos_tau_a = cos(tau_a);
    sin_theta_rc = sin(theta_a + R); /* roll corrected rotation angle */
    cos_theta_rc = cos(theta_a + R); /* roll corrected rotation angle */

    ra->x = xsubt = (cos_theta_rc * sinD * cos_tau_a * sinP
	     + cosD * sin_theta_rc * cos_tau_a
	     -sinD * cosP * sin_tau_a);
    ra->y = ysubt = (-cos_theta_rc * cosD * cos_tau_a * sinP
	     + sinD * sin_theta_rc * cos_tau_a
	     + cosP * cosD * sin_tau_a);
    ra->z = zsubt = (cosP * cos_tau_a * cos_theta_rc
	     + sinP * sin_tau_a);

    ra->rotation_angle = theta_t = atan2(xsubt, zsubt);
    ra->tilt = tau_t = asin(ysubt);
    lambda_t = atan2(xsubt, ysubt);
    ra->azimuth = fmod(lambda_t + T, TWOPI);
    ra->elevation = phi_t = asin(zsubt);

# else

    rtrack = rhead +rdrift;
    ra->rotation_angle = RADIANS(asib->rotation_angle +cfac->rot_angle_corr
			+asib->roll +cfac->roll_corr);
    ra->tilt = RADIANS(asib->tilt +cfac->tilt_corr);
    azr = rhead +PI -(PIOVR2+ra->tilt)*sin(ra->rotation_angle);
    
    rphi = (ra->rotation_angle + PI);
    sin_pitch = sin(rpitch); cos_pitch = cos(rpitch);
    sin_tilt = sin(ra->tilt); cos_tilt = cos(ra->tilt);
    cos_drift = cos(rdrift);
    cos_phi = cos(rphi);
    
    cos_psi = cos_pitch*cos_drift*sin_tilt
	  + sin_pitch*cos_drift*cos_tilt*cos_phi
		- sin(rdrift)*cos_tilt*sin(rphi);
    /* psi is the angle between the beam and the
     * horizontal component of the aircraft velocity vector
     */
    ra->psi = acos(cos_psi);

    sin_el = sin_tilt*sin_pitch-cos_tilt*cos_pitch*cos_phi;
    ra->elevation = asin(sin_el);
    cos_el = cos(ra->elevation);
    cos_az = (sin_tilt-sin_pitch*sin_el)/(cos_pitch*cos_el);

    if(fabs(cos_az) <= 1.0 )
	  az = acos(cos_az);
    else {
	az = cos_az > 1.0 ? 0 : PI;
	printf( "cos_az:%f\n", cos_az );
    }

    if(ra->rotation_angle >= PI && ra->rotation_angle < TWOPI ) {
	az = TWOPI -az;
    }
    az -= rdrift;

    /* compute x,y,z */
    ra->x = sin(az)*cos_el;
    ra->y = cos(az)*cos_el;
    ra->z = sin_el;

    if( ra->elevation >= 0 ) {
	ra->z = fabs((double)ra->z);
    }
    else {
	ra->z = -fabs((double)ra->z);
    }
    if(az >= 0 && az <= PIOVR2) {
	ra->x = fabs((double)ra->x);
	ra->y = fabs((double)ra->y);
    }
    else if( az > PIOVR2 && az <= PI ) {
	ra->x = fabs((double)ra->x);
	ra->y = -fabs((double)ra->y);
    }
    else if( az > PI && az <= PI+PIOVR2 ) {
	ra->x = -fabs((double)ra->x);
	ra->y = -fabs((double)ra->y);
    }
    else if( az > PI+PIOVR2 && az < TWOPI ) {
	ra->x = -fabs((double)ra->x);
	ra->y = fabs((double)ra->y);
    }
    ra->azimuth = fmod(az+rtrack, TWOPI);
    d = ra->x*ra->x + ra->z*ra->z;
    bigR = sqrt(d);
    ra->tilt = atan2((double)ra->y, bigR);
    ra->rotation_angle = atan2((double)ra->x, (double)ra->z);

    return;
# endif
}
/* c------------------------------------------------------------------------ */

char *dd_radar_name(dds)
  struct dds_structs *dds;
{
    int i;
    static char name[12];
    char *a;
    struct radar_d *radd=dds->radd;
    
    strncpy( name, radd->radar_name, 8 );

    /* remove trailing blanks or extra nulls */
    for(i=8,a=name+7; i && (*a == ' ' || *a == '\0'); i--, a-- ) ;
    *(name+i) = '\0';

    /* replace any blanks in the name with underscores */
    for(a=name; *a; a++ ) {
	if(*a == ' ' || *a == '/')
	      *a = '_';
    }
    return( name );
}
/* c------------------------------------------------------------------------ */

char *dd_radar_namec(name0)
  char *name0;
{
    /* try and cleanup the radar name
     */
    int ii, nn=8;		/* process at most 8 characters */
    static char name[12];
    char *a, *b, *c;
    
    a = name0;
    for(; nn && *a == ' '; nn--, a++); /* in case there are
					* leading blanks */
    c = name;
    for(; nn && *a; nn--, *c++ = *a++);
    for(; name < c && *(c-1) == ' '; c--); /* remove trailing blanks */
    *c = '\0';
    for(c=name; *c; c++)
	  if(*c == ' ' || *c == '/')
		*c = '_';	/* substitute for blanks or slashes */
    return(name);
}
/* c------------------------------------------------------------------------ */

void dd_radar_selected( radar_name, radar_num, difs)
  char *radar_name;
  int radar_num;
  struct dd_input_filters *difs;
{
    int ii, jj, kk, nt;
    char *a, *b, *c, str[256], *get_tagged_string(), radar[16];
    char *str_ptrs[32];
    char *dd_whiteout(), *dd_delimit(), *str_terminate();

    if(!(a=get_tagged_string("SELECT_RADARS"))) {
	for(ii=0; ii < MAX_SENSORS; ii++ )
	    difs->radar_selected[ii] = YES;
	return;
    }
    str_terminate(radar, radar_name, 8);
    strcpy(str, a);
    nt = dd_tokens(str, str_ptrs);
    a = dd_whiteout(a);

# ifdef notyet
    for(ii=0; ii < nt; ii++) {
	if(strstr(radar,str_ptrs[ii])) {
	    difs->radar_selected[radar_num] = YES;
	    difs->num_selected_radars++;
	}
    }
# else
    while(*a) {
	b = dd_delimit(a);
	/* see if we can construct a select string */
	for(c=str; a < b; *c++ = *a++);
	if(c == str) break;
	*c = '\0';
	if(strstr(radar,str)) {
	    difs->radar_selected[radar_num] = YES;
	    difs->num_selected_radars++;
	}
	a = dd_whiteout(a);
    }
# endif
}
/* c------------------------------------------------------------------------ */

void dd_range_gate( dgi, req_range, gate, val )
  DGI_PTR dgi;
  int *gate;
  float *req_range, *val;
{
    /* return the gate and range of the gate nearest the requested range
     */
    DDS_PTR dds=dgi->dds;
    struct cell_d *celv=dds->celvc;
    int i, j, parameter_num=0;
    double diff=1.e11, last_d=1.e11;
    double d, r= *req_range; /* assume meters */
    double deltag;

    if( r <= celv->dist_cells[0]) {
	*gate = 1;
	*val = celv->dist_cells[0];
	return;
    }
    if( r >= celv->dist_cells[celv->number_cells-1]) {
	*gate = celv->number_cells;
	*val = celv->dist_cells[celv->number_cells-1];
	return;
    }
# ifdef obsolete
    *gate = 1 +fp_bin_search( celv->dist_cells, celv->number_cells, r );
# else
    *gate = 1 + dd_cell_num(dgi->dds, parameter_num, r);
# endif
    *val = celv->dist_cells[*gate-1];

    return;
}
/* c------------------------------------------------------------------------ */

struct dd_ray_sector *
dd_ray_coverage(dgi, rat, rn, expansion)
  DGI_PTR dgi;
  struct rot_ang_table *rat;
  int rn;			/* ray number (starts at 1) */
  float expansion;
{
    /* this routine returns the sector in degrees covered by
     * the request ray
     */
    struct rot_table_entry *ray0, *ray1, *ray2, *return_rotang1();
    static struct dd_ray_sector rs;
    double d, angdiff(), sector;
    
    ray0 = (struct rot_table_entry *)((char *)rat +
					rat->first_key_offset);
    if(rn == 1) {
	ray1 = ray0 +1;
	rs.sector = angdiff(ray0->rotation_angle, ray1->rotation_angle);
	rs.angle0 = FMOD360(ray0->rotation_angle - .5 * rs.sector +360.);
	rs.angle1 = FMOD360(ray0->rotation_angle + .5 * rs.sector +360.);
	rs.rotation_angle = ray0->rotation_angle;
    }
    else if(rn >= rat->num_rays) {
	rn = rat->num_rays;
	ray0 += rn-2;
	ray1 = ray0 +1;
	rs.sector = angdiff(ray0->rotation_angle
		    , ray1->rotation_angle);
	rs.angle0 = FMOD360(ray1->rotation_angle - .5 * rs.sector +360.);
	rs.angle1 = FMOD360(ray1->rotation_angle + .5 * rs.sector +360.);
	rs.rotation_angle = ray1->rotation_angle;
    }
    else {
	ray0 += rn-2;
	ray1 = ray0 +1;
	ray2 = ray0 +2;
	
	rs.angle0 = ray0->rotation_angle + .5 * angdiff
	      (ray0->rotation_angle, ray1->rotation_angle);
	rs.angle1 = ray1->rotation_angle + .5 * angdiff
	      (ray1->rotation_angle, ray2->rotation_angle);
	
	rs.sector = angdiff(rs.angle0, rs.angle1);
	rs.rotation_angle = ray1->rotation_angle;
    }
    if(expansion) {
	if(expansion < .9) {
	    sector = rs.sector < 0 ? -expansion : expansion;
	}
	else
	      sector = expansion*rs.sector;
	rs.angle1 = FMOD360(rs.angle0 + sector +360.);
    }
    return(&rs);
}
/* c------------------------------------------------------------------------ */

void dd_razel_to_xyz(p0)
  struct point_in_space *p0;
{
    /* routine to calculate (x,y,z) (from az,el,rng)
     *
     */
    double d, r, rxy, theta, phi;

    phi = RADIANS(p0->elevation);
    p0->z = p0->range * sin(phi);
    rxy = p0->range * cos(phi);
    theta = RADIANS(CART_ANGLE(p0->azimuth));
    p0->x = rxy * cos(theta);
    p0->y = rxy * sin(theta);
}
/* c------------------------------------------------------------------------ */

void dd_reset_d_limits(at)
  struct d_limits **at;
{
    if(!(*at)) {
	*at = (struct d_limits *)malloc(sizeof(struct d_limits));
	memset(*at, 0, sizeof(struct d_limits));
    }
    (*at)->lower = -MAX_FLOAT;
    (*at)->upper =  MAX_FLOAT;
}
/* c------------------------------------------------------------------------ */

int dd_return_id_num(name)
  char *name;
{
    char *a, *b, str[16], *str_terminate();

    a = str_terminate(str, name, 8);
    b = "DBZ";
    if(strstr(a, b)) {
	return(DBZ_ID_NUM);
    }
    b = "SW";
    if(strstr(a, b)) {
	return(SW_ID_NUM);
    }
    b = "VR";
    if(strstr(a, b)) {
	return(VR_ID_NUM);
    }
    b = "NCP";
    if(strstr(a, b)) {
	return(NCP_ID_NUM);
    }
    b = "DZ";
    if(strstr(a, b)) {
	return(DZ_ID_NUM);
    }
    b = "UZ";
    if(strstr(a, b)) {
	return(DZ_ID_NUM);
    }
    b = "UDBZ";
    if(strstr(a, b)) {
	return(DZ_ID_NUM);
    }
    b = "VE";
    if(strstr(a, b)) {
	return(VE_ID_NUM);
    }
    b = "VG";
    if(strstr(a, b)) {
	return(VG_ID_NUM);
    }
    b = "VU";
    if(strstr(a, b)) {
	return(VU_ID_NUM);
    }

    return(0);
}
/* c------------------------------------------------------------------------ */

struct rot_table_entry *
dd_return_rotang1(rat)
  struct rot_ang_table *rat;
{
    struct rot_table_entry *ray1;

    ray1 = (struct rot_table_entry *)((char *)rat + rat->first_key_offset);
    return(ray1);
}
/* c------------------------------------------------------------------------ */

double
dd_roll(dgi)
  struct dd_general_info *dgi;
{
    double d=dgi->dds->asib->roll;
    if(dd_isnanf(d))
	  return((double)0);
    else
	  return(d +dgi->dds->cfac->roll_corr);
}
/* c------------------------------------------------------------------------ */

int dd_rotang_seek(rat, rotang)
  struct rot_ang_table *rat;
  float rotang;
{
    /* returns the index of the ray closest to the requested
     * rotation angle
     */
    int i, j, k, n=rat->ndx_que_size, ang_ndx;
    long *iptr;
    char *c=(char *)rat;
    struct rot_table_entry *entry1;
    double diff1, diff2;

    if(rat->num_rays < 1)
	  return(-1);

    for(; rotang < 0; rotang+=360.);
    for(; rotang > 360.; rotang-=360.);

    iptr = (long *)(c + rat->angle_table_offset);
    ang_ndx = rotang*rat->angle2ndx;

    /* get ray index(s) from the angle index table
     */
    if(*(iptr+ang_ndx) >= 0)	/* found one */
	  return(*(iptr+ang_ndx));

    /* otherwise we have an empty slot
     * look for an entries on either side of the rotang
     */
    j = DEC_NDX(ang_ndx, n);
    for(i=0; i < n && *(iptr+j) < 0; i++) {
	j = DEC_NDX(j,n);
    }
    j = *(iptr+j);		/* this is the ray index now */

    k = INC_NDX(ang_ndx, n);
    for(i=0; i < n && *(iptr+k) < 0; i++) {
	k = INC_NDX(k,n);
    }
    k = *(iptr+k);

    /* we now have one entry on either side of the requested
     * rotation angle
     */
    entry1 = (struct rot_table_entry *)((char *)rat +
					rat->first_key_offset);
    diff1 = fabs((double)(rotang-(entry1+j)->rotation_angle));
    diff2 = fabs((double)(rotang-(entry1+k)->rotation_angle));
    i = diff1 < diff2 ? j : k;
    return(i);
}
/* c------------------------------------------------------------------------ */

void dd_rotang_table(dgi, func)
  DGI_PTR dgi;
  int func;
{
    struct rot_ang_table *rat=dgi->rat;
    struct rot_table_entry *next;
    int inc_entries=480, size;
    static int count=0, trip=170670;
    char *c;
    int i, j, k, mm, ndx, mark, angle_ndx_size=480, first=NO, glitch=NO;
    long *lp;
    float f, rot, angle;
    double dd_rotation_angle();
    unsigned long nan = 0x7fffff, *langle;

    /* c...mark */

    if(++count >= trip) {
	mark = 0;
    }
    if(!rat || rat->num_rays >= dgi->max_rat_entries) {
	/* Initialize or resize rat
	 */
	dgi->max_rat_entries += inc_entries;
	mm = sizeof(struct rot_ang_table);
	mm = ((mm -1)/8 +1) * 8; /* start tables on an 8 byte boundary */

	size = mm + angle_ndx_size * sizeof(long) +
	  dgi->max_rat_entries * sizeof(struct rot_table_entry);

	if(!rat) {
	    first = YES;
	    c = (char *)malloc(size);
	    memset(c, 0, size);
	}
	else {
	    c = (char *)realloc((char *)rat, size);
	}
	if(!c) {
	    printf("Unable to malloc rotang table\n");
	    exit(1);
	}
	dgi->rat = rat = (struct rot_ang_table *)c;
	strncpy(rat->name_struct, "RKTB", 4);
	rat->sizeof_struct = size;

	rat->angle_table_offset = mm;
	dgi->rat_angle_ndx1 = (long *)(c + rat->angle_table_offset);
	rat->angle2ndx = (float)angle_ndx_size/360.;
	rat->ndx_que_size = angle_ndx_size;

	rat->first_key_offset = rat->angle_table_offset +
	      rat->ndx_que_size * sizeof(long);
	dgi->rat_entry1 = (struct rot_table_entry *)
	      (c + rat->first_key_offset);
	mark = 0;
    }

    if(func == RESET || first) {
	/* reset angle index table */
	for(i=0; i < angle_ndx_size; *(dgi->rat_angle_ndx1 + i++) = -1);
	if(func == RESET) {
	    rat->num_rays = 0;
	    return;
	}
    }

    /* put this entry in the table */

    ndx = rat->num_rays++;

    if((angle = dd_rotation_angle(dgi)) < 0) {
	if((angle = angle +360.) < 0)
	      i = 0;
	else
	      i = angle * rat->angle2ndx;
    }
    else if(angle >= 360.)
	  i = angle_ndx_size -1;
    else
	  i = angle * rat->angle2ndx;

    if(i >= angle_ndx_size || i < 0) {
	i = 0;
    }
    next = dgi->rat_entry1 + ndx;
    next->rotation_angle = angle;
    next->offset = dgi->ray_que->disk_offset;
    next->size = dgi->ray_que->sizeof_ray;
    *(dgi->rat_angle_ndx1+i) = ndx;
}
/* c------------------------------------------------------------------------ */

double
dd_rotation_angle(dgi)
  struct dd_general_info *dgi;
{
# define Not_Track_Relative

    double d_rot, dd_azimuth_angle();


    switch(dgi->dds->radd->scan_mode) {

    case AIR:
# ifdef Not_Track_Relative
	d_rot = FMOD360(dgi->dds->asib->rotation_angle +
			dgi->dds->cfac->rot_angle_corr +
			dd_roll(dgi));
# else
	d_rot = FMOD360(DEGREES(dgi->dds->ra->rotation_angle)+360.);
# endif
	break;

    case RHI:
	d_rot = FMOD360((450.-(dgi->dds->ryib->elevation
			       + dgi->dds->cfac->elevation_corr)));
	break;

    case TAR:
	d_rot = dgi->dds->radd->radar_type != GROUND ?
	      FMOD360(dgi->dds->asib->rotation_angle +
			dgi->dds->cfac->rot_angle_corr) :
			      dd_azimuth_angle(dgi);
	break;

    default:
	d_rot = dd_azimuth_angle(dgi);
	break;
    }
    return(d_rot);
}
/* c------------------------------------------------------------------------ */

void dd_rng_info( dgi,  g1, m, rngs, ng )
  DGI_PTR dgi;
  int *g1, *m, *ng;
  float *rngs;
{
    DDS_PTR dds=dgi->dds;
    /* routine to return "m" range values */
    struct cell_d *celv=dds->celvc;
    int i=(*g1-1), n, nc;

    nc = celv->number_cells;

    if((n=i+(*m)) > nc)
	  n = nc;

    for(; i < n; i++)
	  *rngs++ = celv->dist_cells[i]; /* return in meters */

    *ng = nc;
}
/* c------------------------------------------------------------------------ */

void
dd_set_uniform_cells(dds)
  struct dds_structs *dds;
{
    /* set up to do quick calculation of a cell number for a given range
     * the assumption here is that the original cell vector contains
     * data that is non-uniformly spaced
     */
    struct cell_d *celv=dds->celvc;
    int ndx=0, nc, just_the_cell_count=YES;
    int nu;
    float cell_spacing, cx, r0, r;
    void dd_uniform_cell_spacing();


    /* the lowest resolution cell spacing is based on the
     * distance between the first two cells
     */
    dds->uniform_cell_count[ndx] = 0;
    dds->rcp_uniform_cell_spacing[ndx] = 0;

    nc = celv->number_cells-1;
    if((nc = celv->number_cells-1) < 1) {
       printf("\n**** Bad cell count info in dd_set_uniform_cells(dds)\n");
       return;
    }
    cell_spacing = celv->dist_cells[1]
	  - celv->dist_cells[0];
    if(cell_spacing <= 0) {
       printf("\n**** Bad cell spacing info in dd_set_uniform_cells(dds)\n");
       return;
    }
    r0 = celv->dist_cells[0];
    r = celv->dist_cells[nc] -r0;

    if((nc = 1.5 + r/cell_spacing) > 32000) {
       printf("\n**** REALLY Bad cell count info in dd_set_uniform_cells(dds)\n");
       return;
    }
    
    /*
     * make sure there is space for it
     */
    if(!dds->uniform_cell_lut[ndx] || dds->uniform_cell_count[ndx] < nc) {
	if(dds->uniform_cell_lut[ndx])
	      free(dds->uniform_cell_lut[ndx]);
	dds->uniform_cell_lut[ndx] = (int *)malloc(nc*sizeof(int));
	memset(dds->uniform_cell_lut[ndx], 0, nc*sizeof(int));
	dds->uniform_cell_count[ndx] = nc;
    }
    /* now really create a table of equivalent cell numbers
     * to the uniform value
     */
    dd_uniform_cell_spacing(celv->dist_cells
			    , celv->number_cells
			    , cell_spacing
			    , dds->uniform_cell_lut[ndx]
			    , r0, nc);

    dds->rcp_uniform_cell_spacing[ndx] = 1./cell_spacing;
    dds->uniform_cell_zero[ndx] = r0;
    return;
}
/* c------------------------------------------------------------------------ */

void dd_site_name(proj_name, site_name)
  char *proj_name, *site_name;
{
    /* add the site name to the end of the project name */

    int m, n;

    m = strlen(site_name) > 8 ? 8 : strlen(site_name);

    for(n=0; n < 20 && *(proj_name+n); n++); /* length of project name */

    if( n > 19-m ) {
	n = 19-m;
	*(proj_name +n) = '\0';
	n++;
    }
    strncpy(proj_name+n, site_name, m);
    if(m+n < 20)
	  *(proj_name+m+n) = '\0';
}
/* c------------------------------------------------------------------------ */

int dd_src_fid(io_type)
  int *io_type;
{
    char *a, *get_tagged_string();
    int fid;

    if((a=get_tagged_string("SOURCE_DEV"))) {
	/* see if the string contains a physical device name */
	if(dd_itsa_physical_dev(a))
	      *io_type = PHYSICAL_TAPE;
	else
	      *io_type = FB_IO;
	/*
	 * open the file
	 */
	printf( "Input file name: %s\n", a );
	if((fid = open(a, 0 )) < 0) { 
	    printf( "Could not open input file %s  error=%d\n", a, fid);
	    exit(1);
	}
	return(fid);
    }
    printf("No SOURCE_DEV specification!\n");
    exit(1);
}
/* c------------------------------------------------------------------------ */

double
dd_tilt_angle(dgi)
  struct dd_general_info *dgi;
{
    double d_rot;

    switch(dgi->dds->radd->scan_mode) {
	
    case AIR:
	d_rot = DEGREES(dgi->dds->ra->tilt);
	break;

    case RHI:
	d_rot = CART_ANGLE
	      (dgi->dds->ryib->azimuth +dgi->dds->cfac->azimuth_corr);
	break;

    default:
	d_rot = dgi->dds->ryib->elevation +dgi->dds->cfac->elevation_corr;
	break;
    }
    return(d_rot);
}
/* c------------------------------------------------------------------------ */

int dd_tokens(att, str_ptrs)
  char *att, *str_ptrs[];
{
    int nargs=0;
    char *b=att, *dlims=" \t\n"; /* blank, tab, and new line */
    char *strtok();

    for(b=strtok(b, dlims);  b; b = strtok(NULL, dlims)) {
	str_ptrs[nargs++] = b;
    }
    return(nargs);
}
/* c------------------------------------------------------------------------ */

int dd_tokenz(att, str_ptrs, dlims)
  char *att, *str_ptrs[], *dlims; /* token delimiters */
{
    int nargs=0;
    char *b=att;
    char *strtok();

    for(b=strtok(b, dlims); b;  b = strtok(NULL, dlims)) {
	str_ptrs[nargs++] = b;
    }
    return(nargs);
}
/* c------------------------------------------------------------------------ */

void
dd_uniform_cell_spacing(src_cells, num_cells, cell_spacing, cell_lut
			, r0, nuc)
  float src_cells[], cell_spacing, r0;
  int num_cells, *cell_lut, nuc;
{
    /* create a lookup table that facilitates getting from a range
     * in meters to a cell number quickly
     * i.e.
     * uniform_cell_num = (range - r0)/cell_spacing;
     * source_cell_num = *(cell_lut + uniform_cell_num);
     */
    int ii=0, gg=0;
    float rr, d1, d2;
    int *lut=cell_lut;

    if(cell_spacing <=0)
	  return;
    rr = r0;
    d1 = src_cells[0];
    d2 = src_cells[1];

    for(--nuc; ii < nuc; ++ii, ++lut, rr += cell_spacing) {

	if(FABS(rr - d1) > .5 * FABS(d2 - d1)) {
	    d1 = d2;
	    d2 = src_cells[++gg];
	}
	*lut = gg;
    }
    *lut = gg;
    return;
}
/* c------------------------------------------------------------------------ */

char *dd_whiteout(c)
  char *c;
{
    /* advance beyond whitespace */
    for(; c && (*c == ' ' || *c == '\t' || *c == '\n'); c++ );
    return(c);
}
/* c------------------------------------------------------------------------ */

int dd_xy_plane_horizontal(dgi)
  struct dd_general_info *dgi;
{
    int ii, nn;
    
    nn = dgi->dds->radd->radar_type == AIR ? NO : YES;
    return(nn);
}
/* c------------------------------------------------------------------------ */

int fp_bin_search( f, nf, val )
  float *f, val;
  int nf;
{
    /* returns the offset to the nearest value
     */
    int i, j=0;
    int low=0, mid, high=nf-1;

    while(low <= high) {
	mid = (low+high)/2;
	j++;
	if(val < *(f+mid))
	      high = mid-1;
	else
	      low = mid +1;
    }

    if(fabs((double)(*(f+mid)-val)) < fabs((double)(*(f+high)-val)))
	  return(mid);
    else
	  return(low);
}
/* c------------------------------------------------------------------------ */

int dgi_buf_rewind(dgi)
    DGI_PTR dgi;
{
    int mark;

    dgi->ray_num = 0;
    dgi->buf_bytes_left = dgi->file_byte_count = 0;
    if(!dgi->in_buf) {
	dgi->in_buf = (char *)malloc(MAX_READ);
	if(!dgi->in_buf) {
	    printf("Unable to malloc dgi->in_buf for: %s %d\n"
		   , dgi->radar_name, dgi->radar_num);
	    exit(1);
	}
	memset(dgi->in_buf, 0, MAX_READ);
    }
    memset(dgi->in_buf, 0, 8);
    dgi->in_next_block = dgi->in_buf;

    if(mark = lseek(dgi->in_swp_fid, 0L, 0L)) {
	mark = 0;
    }

    return(1);
}
/* c------------------------------------------------------------------------ */

short swap2( ov )		/* swap integer*2 */
  char *ov;
{
    union {
	short newval;
	char nv[2];
    }u;
    u.nv[1] = *ov++; u.nv[0] = *ov++;
    return(u.newval);
}
/* c------------------------------------------------------------------------ */

long swap4( ov )		/* swap integer*4 */
  char *ov;
{
    union {
	long newval;
	char nv[4];
    }u;
    u.nv[3] = *ov++; u.nv[2] = *ov++; u.nv[1] = *ov++; u.nv[0] = *ov++;
    return(u.newval);
}
/* c------------------------------------------------------------------------ */

long xswap4( v_lower, v_upper ) /* assemble an integer*4 variable */
  char *v_lower, *v_upper;
{
    union {
	long newval;
	char nv[4];
    }u;
    u.nv[3] = *v_lower++;
    u.nv[2] = *v_lower++;
    u.nv[1] = *v_upper++;
    u.nv[0] = *v_upper++;
    return(u.newval);
}




/*=========================================================================
/*     below functions are copied from other source code
/*     "solo_add_list_entry"        from dorade_share.c
/*     "get_tagged_string"          from dorade_share.c
/*     "angdiff"                    from dorade_share.c
/*========================================================================= */
/* c------------------------------------------------------------------------ */

double angdiff( a1, a2 )
  float a1, a2;
{
    double d=a2-a1;

    if( d < -270. )
          return(d+360.);
    if( d > 270. )
          return(d-360.);
    return(d);
}
/* c------------------------------------------------------------------------ */

char *get_tagged_string( tag )
  char *tag;
{
    int i;

    for(i=0; i < num_tags; i++ ) {
        if(!strcmp(tag,tags[i])) {
            return(tagged_strings[i]);
        }
    }
    return(0);
}
/* c------------------------------------------------------------------------ */

void solo_add_list_entry(which, entry, len)
  struct solo_list_mgmt *which;
  char *entry;
  int len;
{
    int ii;
    char *a, *c, *str_terminate();

    if(!which)
          return;

    if(!len) len = strlen(entry);

    if(which->num_entries >= which->max_entries) {
        which->max_entries += 30;
        if(which->list) {
            which->list = (char **)realloc
                  (which->list, which->max_entries*sizeof(char *));
        }
        else {
            which->list = (char **)malloc(which->max_entries*sizeof(char *));
        }
        for(ii=which->num_entries; ii < which->max_entries; ii++) {
            *(which->list+ii) = a = (char *)malloc(which->sizeof_entries+1);
            *a = '\0';
        }
    }
    len = len < which->sizeof_entries ? len : which->sizeof_entries;
    c = *(which->list+which->num_entries++);

    if(a=entry) {
        for(; *a && len--; *c++ = *a++);
    }
    *c = '\0';
}
/* c------------------------------------------------------------------------ */

