/*------------------------------------------------------------------------
     text_output.c
     * This file contains the following routines
         text_output_header_info(dgi,dis)

                                            by Yuuki Honda  12.Jun.2000
  ------------------------------------------------------------------------*/
# include "Volume.h"
# include "Comment.h"
# include "RadarDesc.h"
# include "Correction.h"
# include "Parameter.h"
# include "CellVector.h"
# include "CellSpacing.h"
# include "Sweep.h"
# include "Platform.h"
# include "Ray.h"
# include "Pdata.h"
# include "Qdata.h"
# include "FieldParam.h"
# include "super_SWIB.h"

# include "dd_defines.h"
# include "dorade_share.h"
# include "dd_general_info.h"

# include "dd_files.h"
# include "input_limits.h"
# include "input_sweepfiles.h"
# include "dd_stats.h"
# include "function_decl.h"
# include "dgi_func_decl.h"


int text_output_header_info(dgi,dis,fp)
    struct dd_general_info *dgi;
    struct dd_input_sweepfiles_v3 *dis;
    FILE *fp;
{
    DDS_PTR dds=dgi->dds;
    DD_TIME *dts=dds->dts;
    char str[128];
    int  ii,pn;

    fprintf(fp,"Radar Name: %s :\n",dis->usi_top->radar_name);
    fprintf(fp,"Field Name: ");
    for(pn=0; pn < dgi->source_num_parms; pn++) {
      str_terminate(str, dds->parm[pn]->parameter_name, 8);
      fprintf(fp,"%s ",str);
    }
    fprintf(fp,":\n");
    fprintf(fp,"Data/Time: %4d %2d %2d , %2d %2d %2d :\n",
            dds->vold->year,dds->vold->month,dds->vold->day,dds->vold->data_set_hour,
            dds->vold->data_set_minute,dds->vold->data_set_second);
    fprintf(fp,"Number of Sweeps: %d :\n",dis->sweep_count);
    fprintf(fp,"First Gate: %10.5f :\n",dds->celv->dist_cells[0]);
    fprintf(fp,"Volume Space: %10.5f :\n",dds->celv->dist_cells[0]);

    fprintf(fp,"\n");

    fprintf(fp,"Radar Location: Latitude : %f :\n",dds->radd->radar_latitude);
    fprintf(fp,"                Longitude: %f :\n",dds->radd->radar_longitude);
    fprintf(fp,"                Altitude : %f :\n",dds->radd->radar_altitude);

    fprintf(fp,"\n");

    fprintf(fp,"Anthena Height: %f :\n",dds->radd->radar_altitude);
/*
    fprintf(fp,"Anthena Height: %f :\n",dds->asib->altitude_agl);
*/

    fprintf(fp,"\n");

}

/* c------------------------------------------------------------------------ */
int text_output_sweep_info(elmean,dds,fp)
     float elmean;
     DDS_PTR dds;
     FILE *fp;
{

    fprintf(fp,"Elevation Angle: %f :\n",elmean);

    fprintf(fp,"Number of Rays: %d :\n",dds->swib->num_rays);

    fprintf(fp,"Number of Gates: %d :\n",dds->celv->number_cells);

    fprintf(fp,"\n");
}

/* c------------------------------------------------------------------------ */
int text_output_data(imax,jmax,kmax,sptr,dds,fp)
     int imax,jmax,kmax;
     float *sptr;
     DDS_PTR dds;
     FILE *fp;
{
  int ii,jj,kk,ijk,nn=0;
  char str[10];

  fprintf(fp,"Azimuth Angle of each Ray:\n");
  for(jj=1;jj<jmax;jj++){
    ijk=jj*(imax+3)+1;
    fprintf(fp,"%10.3f", *(sptr+ijk));
    if(jj%20==0 || jj==jmax-1) fprintf(fp,"\n");
  }

  fprintf(fp,"Elevation Angle of each Ray:\n");
  for(jj=1;jj<jmax;jj++){
    ijk=jj*(imax+3)+2;
    fprintf(fp,"%10.3f", *(sptr+ijk));
    if(jj%20==0 || jj==jmax-1) fprintf(fp,"\n");
  }

  fprintf(fp,"Observation Time of each Ray:\n");
  for(jj=1;jj<jmax;jj++){
    ijk=jj*(imax+3);
    fprintf(fp,"%10.3f", *(sptr+ijk));
    if(jj%20==0 || jj==jmax-1) fprintf(fp,"\n");
  }

  fprintf(fp,"\n");

  for(kk=0;kk<kmax;kk++){

    /* Parameter Name */
    str_terminate(str, dds->parm[kk]->parameter_name, 8);
    fprintf(fp,"Parameter Name: %s\n",str);

    nn=0;
    for(jj=1;jj<jmax;jj++){
      for(ii=3;ii<imax;ii++){
	ijk=ii+jj*(imax+3)+kk*(imax+3)*(jmax+1);
	fprintf(fp,"%8.2f",*(sptr+ijk));
	if((++nn)%20==0) fprintf(fp,"\n");
      }
    }
    fprintf(fp,"\n");
  }
}
